"""
 *  This file is part of chiplotle.
 *
 *  http://music.columbia.edu/cmc/chiplotle
"""

from chiplotle.plotters.drawingplotter import _DrawingPlotter

class HP7596A(_DrawingPlotter):
   def __init__(self, ser, **kwargs):
      self.allowedHPGLCommands = tuple(['\x1b.', 'AA','AF','AH','AP','AR',
         'AS','BF','BL','CA','CC','CI','CM','CP','CS','CT','CV','DC','DF',
         'DI','DL','DP','DR','DS','DT','EA','EC','EP','ER','ES','EW','FP',
         'FR','FS','FT','GC','GM','GP','IC','IM','IN','IP','IV','IW','KY',
         'LB','LO','LT','NR','OA','OB','OC','OD','OE','OF','OG','OH','OI',
         'OK','OL','OO','OP','OS','OT','OW','PA','PB','PD','PG','PM','PR',
         'PT','PU','RA','RL','RO','RP','RR','SA','SC','SG','SI','SL','SM',
         'SP','SR','SS','TL','UC','UF','VA','VN','VS','WD','WG','XT','YT'])

      _DrawingPlotter.__init__(self, ser, **kwargs)
      self.type = "HP7596A"




from chiplotle import *
from chiplotle.hpgl.commands import PA

def test_get_centroid_01( ):
   g = [PA([(0, 0), (10, 10), (0, 10), (10, 0)])]
   t = hpgltools.get_centroid(g)
   assert t == Coordinate(5, 5)





from chiplotle.geometry.core.shape import _Shape

class FormatDecorator(object):
   '''FormatDecorators know how to wrap Shape objects with additional
   formatting stuff.
   '''
   
   @property
   def _subcommands(self):
      raise NotImplementedError


   ## overrides ##

   def __call__(self, shape):
      if not isinstance(shape, _Shape):
         raise TypeError('Must be a Shape.')
      #shape.formatters.append(self)
      shape.formatters.add(self)




from chiplotle.geometry.core.path import Path
from chiplotle.geometry.core.group import Group
from chiplotle.geometry.shapes.line import line

def grid(width, height, width_divisions,height_divisions):
   '''Rectangular grid. 

   - `width` : ``int`` or ``float``, width of the rectangle.
   - `height` : ``int`` or ``float``, height of the rectangle.
   - `width_divisions` : ``int``, number of horizontal equidistant partitions.
   - `height_divisions` : ``int``, number of vertical equidistant partitions.
   
   '''

   ul_x = width
   bl_x = ul_x
   ur_x = ul_x + width

   ul_y = height
   ur_y = ul_y
   bl_y = ul_y - height
   
   x_step_size = width / width_divisions
   y_step_size = height / height_divisions

   g = Group()
   
   ## add horizontal lines
   for i in range(height_divisions + 1):
      step_y = y_step_size * i
      l = line((ul_x, ul_y - step_y), (ur_x, ur_y - step_y))
      g.append(l)
   ## add vertical lines

   for i in range(width_divisions + 1):
      step_x = x_step_size * i
      l = line((ul_x + step_x, ul_y), (bl_x + step_x, bl_y))
      g.append(l)

   return g

## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   gr = grid(1000, 2000, 10, 20)
   assert isinstance(gr, Group)
   print gr.format
   io.view(gr)



from chiplotle.hpgl.commands import LT
from chiplotle.core.interfaces.formatdecorator import FormatDecorator

class LineType(FormatDecorator):
   
   __doc__ = LT.__doc__

   def __init__(self, linetype=None, length=4):
      FormatDecorator.__init__(self)
      self.linetype  = linetype
      self.length    = length

   @property
   def _subcommands(self):
      return [LT(self.linetype, self.length)]


## DEMO
if __name__ == '__main__':
   from chiplotle import *

   lines = Group()
   for i in range(10):
      dot_space = i / 2.0
      line_space = i * 400

      lt = formatters.LineType(1, dot_space)
      l = line((0, line_space), (3000, line_space))
      lt(l)

      lb = label(str(dot_space), .1, .1)
      offset(lb, (-10, line_space))

      lines.append(l)
      lines.append(lb)

   formatters.Pen(1)(lines)
   offset(lines, (0, 1000))
   io.view(lines)
   #p = plottertools.instantiate_plotters()[0]
   #p.write(lines)



from chiplotle.geometry.core.path import Path
from chiplotle.tools.mathtools.catmull_interpolation \
   import catmull_interpolation


def catmull_path(points, interpolation_count=50):
   '''Path with Catmull-Rom spline interpolation.'''

   path_points = catmull_interpolation(points, interpolation_count)
   return Path(path_points)



## DEMO
if __name__ == '__main__':
   from chiplotle.tools import io
   from chiplotle.geometry.shapes.cross import cross
   from chiplotle.geometry.core.group import Group
   from chiplotle.geometry.transforms.offset import offset
   import random

   points = [ ]
   for i in range(10):
      x, y = random.randint(-100, 100), random.randint(-100, 100)
      points.append((x, y))

   crosses = [ ]
   for point in points:
      c = cross(15, 15)
      offset(c, point)
      crosses.append(c)

   path = catmull_path(points)
   
   g = Group([path] + crosses)
   io.view(g)




from chiplotle.hpgl.abstract.hpgl import _HPGL

class _HPGLEscape(_HPGL):

   _escape = chr(27)


   ## PUBLIC PROPERTIES ##

   @property
   def format(self):
      return '%s.%s' % (self._escape, self._name)


   ## OVERRIDES ##

   def __repr__(self):
      return 'Escape(%s, %s)' % (repr(self._escape), self._name)





def sniff_ports_for_plotters(ports):
   '''Sniffs all given serial `ports` in search for pen plotters.
   The function returns a dictionary of plotter names found, with keys
   being the port name or address and values the plotter names.'''
   from chiplotle.tools.serialtools import what_plotter_in_port

   plotters_found = { }
   #for k, port in scan_serial_ports( ).items( ): 
   for port in ports: 
      plotter_name = what_plotter_in_port(port)
      if plotter_name:
         plotters_found[port] = plotter_name
  
   return plotters_found 



from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.path import Path
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.transforms.transformvisitor import TransformVisitor
import random

def noise(shape, value):
   '''Distort shape by adding noise.

   - `value` can be a scalar or a tuple (x, y) that sets the range of the 
      noise for the x and y coordinates.
   '''
   def noisify(coords, value):
      try:
         x, y = value
      except TypeError:
         x = y = value
      result = [ ]
      for point in coords:
         x_wiggle = random.randrange(-x, x)
         y_wiggle = random.randrange(-y, y)
         xy = point + Coordinate(x_wiggle, y_wiggle)         
         result.append(xy)
      return CoordinateArray(result)

   t = TransformVisitor(noisify)
   t.visit(shape, value)
      
      
   
## RUN DEMO CODE
if __name__ == "__main__":
   from chiplotle.geometry.shapes.circle import circle
   from chiplotle.tools import io
   c1 = circle(1000, 100)
   c2 = circle(800, 100)
   noise(c1, 90)
   c1 += Coordinate(1000, 1000)
   g = Group([c1, c2])
   noise(g, 60)
   io.view(g)



import os
import sys


def _open_file(file_name, application = None):
   '''Generic cross-platform file opener.

   - `file_name` is the file to open, including path.
   - `application` is the application to use to open the file;
      must be a string or None. If not given, the function will use a 
      generic OS dependent file openner.
   '''

   if os.name == 'nt':
      os.startfile(file_name)
   else:
      if sys.platform.lower( ) == 'linux2':
         viewer = application or 'xdg-open'
      else:
         viewer = application or 'open'
      os.system('%s %s &' % (viewer, file_name))




from chiplotle import *
import py.test

def test_pen_01( ):
   '''Pen must take one argument at least.'''

   assert py.test.raises(TypeError, 't = Pen( )')


def test_pen_02( ):
   t = Pen(2)

   assert t.number == 2
   assert t.acceleration == None
   assert t.force == None
   assert t.velocity == None
   assert t.thickness == None

   assert t.format == 'SP2;' 


def test_pen_03( ):
   t = Pen(2, 3, 4, 5, 0.1)

   
   assert t.number == 2
   assert t.acceleration == 5
   assert t.force == 4
   assert t.velocity == 3
   assert t.thickness == 0.1

   assert t.format == 'SP2;AS5,2;FS4,2;VS3,2;PT0.10;'


def test_pen_04( ):
   t = Pen(2, 3, 4)

   
   assert t.number == 2
   assert t.acceleration == None
   assert t.force == 4
   assert t.velocity == 3
   assert t.thickness == None

   assert t.format == 'SP2;FS4,2;VS3,2;'




from chiplotle.hpgl.commands import PM, EP, FP, FT, SP
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.core.path import Path
from chiplotle.tools.hpgltools.convert_coordinates_to_hpgl_absolute_path \
   import convert_coordinates_to_hpgl_absolute_path

class Polygon(Path):
   '''A closed path.'''

   def __init__(self, points, filled=False):  
      Path.__init__(self, points)
      self.filled = filled

   @property
   def _preformat_points(self):
      '''Points (coordinates) ready for formatting (conversion to HPGL).'''
      coords = self.points[:]
      coords.append(coords[0])
      return CoordinateArray(coords)




if __name__ == '__main__':
   from chiplotle import io

   p = Polygon([(0, 0), (2000, 0), (1000, 1000), (0, 500)], 0)
   print p.points
   print p._preformat_points

   io.view(p)




def factors(n):
   '''Return factors of positive `n` in increasing order::

      >>> mathtools.factors(84)
      [1, 2, 2, 3, 7]

   ::

      >>> for n in range(10, 20):
      ...   print n, mathtools.factors(n)
      ... 
      10 [1, 2, 5]
      11 [1, 11]
      12 [1, 2, 2, 3]
      13 [1, 13]
      14 [1, 2, 7]
      15 [1, 3, 5]
      16 [1, 2, 2, 2, 2]
      17 [1, 17]
      18 [1, 2, 3, 3]
      19 [1, 19]

   '''

   if not isinstance(n, (int, long)):
      raise TypeError
   if n <= 0:
      raise ValueError

   d = 2
   factors = [1]
   while 1 < n:
      if n % d == 0:
         factors.append(d)
         n = n / d
      else:
         d = d + 1
   return factors



from chiplotle.tools.serialtools.virtual_serial_port import VirtualSerialPort
from chiplotle.tools.plottertools import instantiate_plotter_from_id
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.core.cfg.get_config_value import get_config_value
from chiplotle import plotters

def instantiate_virtual_plotter(left_bottom = Coordinate(0,0), 
    right_top = Coordinate(10320, 7920), 
    type=None):
   '''
   Instantiates a virtual plotter with 8.5x11" (ANSI A) paper.
   If you have a default plotter defined in your config.py file
   we will use that plotter definition file (ignoring the serial
   port setting).
   '''

   which_plotter = type
   
   if type is None:
      map = get_config_value('serial_port_to_plotter_map')
      ## if user has set fixed port to plotter mapping...

      if map is not None:
         for k, v in map.items( ):
            which_plotter = v
      else:
         which_plotter = "Plotter"
         
   ser = VirtualSerialPort(left_bottom, right_top)
   plotter = instantiate_plotter_from_id(ser, which_plotter)
   #plotter = getattr(plotters, which_plotter)(ser)
   print "\nInstantiated plotter %s:" % plotter
   coords = plotter.margins.soft.all_coordinates
   print "   Drawing limits: (left %s; bottom %s; right %s; top %s)" % coords
   print "   Buffer Size: %s" % plotter.buffer_size

         
   return plotter





from chiplotle import *
from py.test import raises


def test_coordinatearray__add__01( ):
   '''Two CoordinateArrays of the same size can be added.'''
   a = CoordinateArray([(1, 2), (3, 4)])
   b = CoordinateArray([(1, 1), (2, 2)])
   t = a + b
   assert isinstance(t, CoordinateArray)
   assert t is not a
   assert t is not b
   assert t == CoordinateArray([(2, 3), (5, 6)])


def test_coordinatearray__add__02( ):
   '''Two CoordinateArrays of different length cannot be added.'''
   a = CoordinateArray([(1, 2), (3, 4)])
   b = CoordinateArray([(1, 1), (2, 2), (3, 3)])
   assert raises(ValueError, 't = a + b')


def test_coordinatearray__add__03( ):
   '''A CoordinateArray and an int cannot be added.'''
   a = CoordinateArray([(1, 2), (3, 4)])
   assert raises(TypeError, 't = a + 2')


def test_coordinatearray__radd__04( ):
   '''An int and a CoordinateArray cannot be added.'''
   a = CoordinateArray([(1, 2), (3, 4)])
   assert raises(TypeError, 't = 2 + a')


def test_coordinatearray__add__05( ):
   '''A CoordinateArray and a float cannot be added.'''
   a = CoordinateArray([(1, 2), (3, 4)])
   assert raises(TypeError, 't = a + 2.3')


def test_coordinatearray__add__06( ):
   '''A float and a CoordinateArray cannot be added.'''
   a = CoordinateArray([(1, 2), (3, 4)])
   assert raises(TypeError, 't = 2.3 + a')


def test_coordinatearray__add__07( ):
   '''A CoordinateArray and a Coordinate can be added.'''
   a = CoordinateArray([(1, 2), (3, 4)])
   t = a + Coordinate(2, 3)
   assert isinstance(t, CoordinateArray)
   assert t is not a
   assert t == CoordinateArray([(3, 5), (5, 7)])


def test_coordinatearray__radd__08( ):
   '''A Coordinate and a CoordinateArray can be added.'''
   a = CoordinateArray([(1, 2), (3, 4)])
   t = Coordinate(2, 3) + a
   assert isinstance(t, CoordinateArray)
   assert t is not a
   assert t == CoordinateArray([(3, 5), (5, 7)])



#!/usr/bin/env python
from chiplotle import *

## HELPER FUNCTIONS ##

def _query_font_size( ):
   char_height = float(raw_input("font height (in cm)? "))
   char_width = float(raw_input("font width (in cm)? "))
   return char_width, char_height
   
def _query_pen( ):
   pen_num = int(raw_input("which pen? "))
   return pen_num


## MAIN FUNCTION ##

def typewriter( ):
   print("***************************")
   print("* CHIPLOTLE TYPEWRITER!!! *")
   print("***************************")
   print("")

   plotter = instantiate_plotters( )[0]

   pen_num = _query_pen( )

   set_size = raw_input("set font size (y/N)? ")

   if set_size.lower( ) == "y":
      cw, ch = _query_font_size( )
      plotter.write(SI(cw, ch))

   plotter.select_pen(pen_num)

   print("")
   print("type at the >>> prompt.")
   print("press RETURN after each line to be plotted.")
   print("enter a blank line for options.")
   print("")

   finished = False

   while finished == False:
      line = raw_input(">>> ")
      if len(line) == 0:
         print("(enter): blank line")
         print("p: select new pen")
         print("s: set new font size")
         print("q: quit")
         response = raw_input("command: ")
         if response == "p":
            pen_num = _query_pen( )
            plotter.select_pen(pen_num)
         elif response == "s":
            cw, ch = _query_font_size( )
            plotter.write(SI(cw, ch))
         elif response == "q":
            finished = True
         else:
            plotter.write(LB("\n\r"))
      else:
         plotter.write(LB(line + "\n\r"))
      
   print("l8r.")


if __name__ == '__main__':
   typewriter( )



from chiplotle import *
from py.test import raises

def test_coordinatearray__init__01( ):
   '''CoordinateArray can be empty.'''
   t = CoordinateArray( )
   assert isinstance(t.coords, list)
   assert isinstance(t.x, tuple)
   assert isinstance(t.y, tuple)
   for e in t:
      assert isinstance(e, Coordinate)


def test_coordinatearray__init__03( ):
   '''CoordinateArray can take a list of tuple pairs.'''
   t = CoordinateArray([(1, 2), (3, 4), (5, 6)])
   assert t[:] == [Coordinate(1, 2), Coordinate(3, 4), Coordinate(5, 6)]
   for e in t:
      assert isinstance(e, Coordinate)


## iadd ##

def test_coordinatearray__iadd__01( ):
   '''In place addition with another CoordinateArray works.'''
   t = CoordinateArray([(1, 2), (3, 4)])
   b = CoordinateArray([(1, 1), (1, 1)])
   tid = id(t)
   t += b
   assert isinstance(t, CoordinateArray)
   assert t is not b
   assert id(t) == tid
   assert t[:] == [Coordinate(2, 3), Coordinate(4, 5)]


def test_coordinatearray__iadd__02( ):
   '''In place addition with a scalar raises.'''
   t = CoordinateArray([(1, 2), (3, 4)])
   tid = id(t)
   b = 1
   assert raises(TypeError, 't += b')


## div ##

#def test_coordinatearray__div__01( ):
#   '''Two CoordinateArrays can be divided.'''
#   a = CoordinateArray([(1, 2), (3, 4)])
#   b = CoordinateArray([(2., 2), (2, 2)])
#
#   t = a / b
#
#   assert isinstance(t, CoordinateArray)
#   assert t.xy == [(.5, 1), (3/2., 2)]


def test_coordinatearray__div__02( ):
   '''A CoordinateArray can be divided by a scalar.'''
   a = CoordinateArray([(1., 2), (4, 8)])
   b = 2

   t = a / b

   assert isinstance(t, CoordinateArray)
   assert t[:] == [Coordinate(.5, 1), Coordinate(2, 4)]


## rdiv ##

## idiv ##

def test_coordinatearray__idiv__02( ):
   '''In place division with another CoordinateArray works.'''
   t = CoordinateArray([(1., 2), (4, 8)])
   tid = id(t)
   b = 2
   t /= b
   assert isinstance(t, CoordinateArray)
   assert id(t) == tid
   assert t[:] == [Coordinate(0.5, 1), Coordinate(2, 4)]


## eq / ne ##

def test_coordinatearray__eq__01( ):
   '''Equality between two CoordinateArrays works.''' 
   a = CoordinateArray([(1, 2), (4, 8)])
   b = CoordinateArray([(1, 2), (4, 8)])
   c = CoordinateArray([(1, 3), (2, 2)])
   assert a == b
   assert a != c


def test_coordinatearray__eq__02( ):
   '''Equality between one CoordinateArray and a list is always false.''' 
   a = CoordinateArray([(1, 2), (4, 8)])
   b = [(1, 2), (4, 8)]
   c = [(1, 3), (2, 2)]
   assert a != b
   assert a != c


def test_coordinatearray__eq__03( ):
   '''Equality between a list and a CoordinateArray is always false.''' 
   a = CoordinateArray([(1, 2), (4, 8)])
   b = [(1, 2), (4, 8)]
   c = [(1, 3), (2, 2)]
   assert b != a
   assert c != a


## __invert__ ##

def test_coordinatearray__invert__01( ):
   t = CoordinateArray([(1, 2), (3, 4)])
   assert ~t == CoordinateArray([(-2, 1), (-4, 3)])


## __neg__ ##

def test_coordinatearray__neg__01( ):
   t = CoordinateArray([(1, 2), (-3, 4)])
   assert -t == CoordinateArray([(-1, -2), (3, -4)])
 
## TODO: pending
## sub ##
## mul ##
## pow ##



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.hpgl import commands as hpgl
import numpy

## TODO: this works, but is quite ugly. Refactor!
def relativize(data):
   '''Converts all absolute coordinate commands (PA, RA, EA, AA)
   into relative commands (PR, RR, ER, AR), so that everything
   has in realtive coordinate values.'''
   ## main body...
   last_position = None
   delta = None
   result = [ ]
   for e in data:
      ## absolute...
      if isinstance(e, (hpgl.PA, hpgl.RA, hpgl.EA, hpgl.AA)):
         if isinstance(e, hpgl.PA):
            ## handle delta...
            if not last_position is None:
               delta = e.xy[0] - last_position
            last_position = e.xy[-1]
         else:
            ## handle delta...
            if not last_position is None:
               delta = e.xy - last_position
            last_position = e.xy
         result += _return_relative_from_absolute(e, delta)

      ## relative...
      elif isinstance(e, (hpgl.PR, hpgl.RR, hpgl.ER, hpgl.AR)):
         if isinstance(e, hpgl.PR):
            coords = [list(c) for c in e.xy]
            if not last_position is None:
               last_position += Coordinate(*numpy.sum(coords, axis = 0))
            else:
               last_position = Coordinate(*numpy.sum(coords, axis = 0))
            result.append(e)
         else:
            last_position = (last_position or 0) + e.xy

      else:
         result.append(e)
   return result


def _return_relative_from_absolute(command, delta):
   result = [ ]
   if isinstance(command, hpgl.PA):
      if delta is not None:
         result.append(hpgl.PR([delta]))
      coords = [list(c) for c in command.xy]
      diff = numpy.diff(coords, axis=0).tolist( )
      if len(diff) > 0:
         result.append(hpgl.PR(diff))
   elif isinstance(command, hpgl.RA) and delta is not None:
      result.append(hpgl.RR(delta))
   elif isinstance(command, hpgl.EA) and delta is not None:
      result.append(hpgl.ER(delta))
   elif isinstance(command, hpgl.AA) and delta is not None:
      result.append(hpgl.AR(delta))
   return result


## Trash...
#def relativize(data):
#   '''Converts all absolute coordinate commands (PA, RA, EA, AA)
#   into relative commands (PR, RR, ER, AR), so that everything
#   has in realtive coordinate values.'''
#   def _return_relative_from_absolute(command, delta):
#      result = [ ]
#      if isinstance(command, hpgl.PA):
#         if delta is not None:
#            result.append(hpgl.PR(delta))
#         diff = numpy.diff(command.xy, axis=0)
#         if len(diff) > 0:
#            result.append(hpgl.PR(diff))
#      elif isinstance(command, hpgl.RA) and delta is not None:
#         result.append(hpgl.RR(delta))
#      elif isinstance(command, hpgl.EA) and delta is not None:
#         result.append(hpgl.ER(delta))
#      elif isinstance(command, hpgl.AA) and delta is not None:
#         result.append(hpgl.AR(delta))
#      return result
#
#   ## main body...
#   last_position = None
#   delta = None
#   result = [ ]
#   for e in data:
#      ## absolute...
#      if isinstance(e, (hpgl.PA, hpgl.RA, hpgl.EA, hpgl.AA)):
#         ## handle delta...
#         if not last_position is None:
#            delta = e.xy[0] - last_position
#         last_position = e.xy[-1]
#         result += _return_relative_from_absolute(e, delta)
#      ## relative...
#      elif isinstance(e, (hpgl.PR, hpgl.RR, hpgl.ER, hpgl.AR)):
#         if not last_position is None:
#            last_position += numpy.sum(e.xy, axis = 0)
#         else:
#            last_position = numpy.sum(e.xy, axis = 0)
#         result.append(e)
#      else:
#         result.append(e)
#   return result
#



from chiplotle.geometry.core.coordinate import Coordinate
import numpy as np

class CoordinateArrayPropertiesMixin(object):

   @property
   def centroid(self):
      coords = set(self._data)
      return sum(coords, Coordinate(0, 0)) / float(len(coords))

   @property
   def center(self):
      '''"center" is defined as being half way between the top/bottom
      and left/right-most points. This will be different from the
      centroid, which takes the distribution of the points into
      consideration.
      '''
      return sum(self.minmax, Coordinate(0, 0)) / 2.0

   @property
   def minmax(self):
      '''Returns the minimum and maximum coordinates.'''
      if len(self._data) == 0:
         return None
      coords= [list(c) for c in self._data]
      mx    = np.max(coords, 0).tolist()
      mn    = np.min(coords, 0).tolist()
      return (Coordinate(*mn), Coordinate(*mx))

   @property
   def difference(self):
      '''Returns the difference between consecutive elements in `self`.
      i.e., first derivative.
      '''
      result = [ ]
      for i in range(len(self) - 1):
         result.append(self[i+1] - self[i])
      return type(self)(result)


   @property
   def cumsum(self):
      '''Returns the cumulative sum.'''
      dimensions = len(self[0])
      result = [Coordinate(*([0] * dimensions))]
      for coord in self:
         result.append(result[-1] + coord)
      return type(self)(result)




from chiplotle.geometry.core.shape import _Shape
from chiplotle.geometry.shapes.line import line
from chiplotle.tools.geometrytools.get_line_intersection \
   import get_line_intersection

def get_shape_intersections(shape1, shape2):
   '''Returns a generator of all intersecting points found in the 
   given shapes.'''

   if not isinstance(shape1, _Shape) or not isinstance(shape2, _Shape):
      raise TypeError

   for i in range(len(shape1.points)-1):
      for j in range(len(shape2.points)-1):
         intersection = get_line_intersection(
            line(shape1.points[i], shape1.points[i+1]), 
            line(shape2.points[j], shape2.points[j+1]))
         if intersection:
            yield intersection


## demo
if __name__ == '__main__':
   from chiplotle import *
   from chiplotle.hpgl.decorators.pen import PenDecorator
   from random import randint

   s1 = bezier_path([randint(0, 200) for  i in range(12)], 1, 14)
   s2 = bezier_path([randint(0, 200) for  i in range(12)], 1, 14)
   PenDecorator(Pen(4))(s1)
   PenDecorator(Pen(2))(s2)

   intersections = get_shape_intersections(s1, s2)
   marks = [ ]
   for i in intersections:
      c = circle(2)
      offset(c, i)
      marks.append(c)

   io.view(Group(marks + [s1, s2]))



from math import sin, cos, pow


def superformula(a, b, m, n1, n2, n3, phi):
   ''' Computes the position of the point on a
   superformula curve.
   Superformula has first been proposed by Johan Gielis
   and is a generalization of superellipse.
   see: http://en.wikipedia.org/wiki/Superformula
   '''

   t1 = cos(m * phi / 4.0) / a
   t1 = abs(t1)
   t1 = pow(t1, n2)

   t2 = sin(m * phi / 4.0) / b
   t2 = abs(t2)
   t2 = pow(t2, n3)

   t3 = -1 / float(n1)
   r = pow(t1 + t2, t3)
   if abs(r) == 0:
      return (0,0)
   else:
      return (r * cos(phi), r * sin(phi))




def flat_list_to_pairs(lst):
   '''Returns a list of pairs created from consecutive elements of the given 
   flat list `lst`.
   
   Example::
      
      >>> flat_list_to_pairs([1,2,3,4,5,6])
      [(1, 2), (3, 4), (5, 6)]
      '''

   if len(lst) % 2 != 0:
      raise ValueError("List must have a pair number of elements.")

   result = [ ]
   for x, y in zip(lst[0:-1:2], lst[1::2]):
      result.append((x, y))
   return result



#!/usr/bin/env python
from chiplotle.tools import *
import sys
import time

def view_hpgl(file):
   '''Convert hpgl to eps and view with default system eps app.'''
   f = io.import_hpgl_file(file)
   io.view(f)
   


if __name__ == '__main__':
   if len(sys.argv) < 2:
      print 'Must give HPGL file to view.\nExample: $ view_hpgl myfile.hpgl'
      sys.exit(2)
   file = sys.argv[1]

   view_hpgl(file) 




def flatten(l, ltypes=(list, tuple)):
   '''
   Flatten nested lists. Returns a 0-depth list or tuple.
   Based on Mike C. Fletcher's flatten.
   '''
   assert isinstance(l, ltypes)
   ltype = type(l)
   l = list(l)
   i = 0
   while i < len(l):
      while isinstance(l[i], ltypes):
         if not l[i]:
            l.pop(i)
            i -= 1
            break
         else:
            l[i:i + 1] = l[i]
      i += 1
   return ltype(l)





import os


def _get_functions_in_module(module_file):
   '''Collects and returns all functions defined in module_file.'''
   result = [ ]
   module_file = module_file.replace(os.sep, '.')
   mod = __import__(module_file, fromlist=['*'])
   for key, value in mod.__dict__.items( ):
      if not key.startswith('_'): ## if not a private function...
         if getattr(value, '__module__', None) == module_file:
            #print '"%s" in module %s ' % (key, module_file)
            result.append(value)
   return result




from chiplotle.hpgl.commands import CP

def test_CP_01( ):
   '''Empty initialization.'''
   t = CP( )
   assert t.spaces is None
   assert t.lines is None
   assert t.format == 'CP;'


def test_CP_02( ):
   '''Initialize with spaces only.'''
   t = CP(2)
   assert t.spaces == 2
   assert t.lines is None
   assert t.format == 'CP2;'


def test_CP_03( ):
   '''Initialize with spaces and lines.'''
   t = CP(2, 1)
   assert t.spaces == 2
   assert t.lines  == 1
   assert t.format == 'CP2,1;'



#from chiplotle.geometry.core.path import Path
from chiplotle.geometry.core.polygon import Polygon

def isosceles(width, height, filled=False):

   tip = (0, height)
   left = (- width / 2.0, 0)
   right = (width / 2.0, 0)

   return Polygon([tip, left, right, tip], filled = filled)



## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   e = isosceles(200, 400, filled=True)
   print e.format
   io.view(e)



from chiplotle import *
from chiplotle.hpgl.commands import *

def test_pr_to_pa_01( ):
   '''PR with one coordinate works.'''
   pr = PR([(1, 2)])
   t = hpgltools.pr_to_pa(pr, (0, 0))
   assert isinstance(t, PA) 
   assert t.xy == CoordinateArray([(1, 2)])
   

def test_pr_to_pa_02( ):
   '''PR with more than one coordinate works.'''
   pr = PR([(1, 2), (1, 1), (-2, 1)])
   t = hpgltools.pr_to_pa(pr)
   assert isinstance(t, PA) 
   assert t.xy == CoordinateArray([(1, 2), (2, 3), (0, 4)])
   

def test_pr_to_pa_03( ):
   '''PR with mpty coordinate array results in empty PA.'''
   pr = PR( )
   t = hpgltools.pr_to_pa(pr)
   assert isinstance(t, PA) 
   assert t.xy == CoordinateArray([ ])


def test_pr_to_pa_04( ):
   '''The function takes the optional argument `starting_position`.'''
   pr = PR([(1, 1), (1, 1)])
   t = hpgltools.pr_to_pa(pr, (10, 20))
   assert isinstance(t, PA) 
   assert t.xy == CoordinateArray([(11, 21), (12, 22)])




def cumsum(lst):
   '''Returns the cumulative sum of the values in `lst`.'''

   try:
      import numpy
      return numpy.cumsum(lst)
   except ImportError:
      r = 0
      result = []
      for n in lst:
         r += n
         result.append(r)
      return result



from chiplotle.hpgl.commands import VS

def test_VS_01( ):
   '''Empty initialization.'''
   t = VS( )
   assert t.vel is None
   assert t.pen is None
   assert t.format == 'VS;'


def test_VS_02( ):
   '''Initialize velocity.'''
   t = VS(1)
   assert t.vel == 1
   assert t.pen is None
   assert t.format == 'VS1;'


def test_VS_03( ):
   '''Initialize velocity and pen.'''
   t = VS(1, 2)
   assert t.vel == 1
   assert t.pen == 2
   assert t.format == 'VS1,2;'


def test_VS_03( ):
   '''Setting vel to None formats correctly.'''
   t = VS(1, 2)
   t.vel = None
   assert t.format == 'VS;'



from chiplotle.geometry.core.polygon import Polygon
from chiplotle.geometry.core.group import Group
from chiplotle.tools.mathtools.lcm import lcm
import math
from fractions import gcd

def star_crisscross(width, height, num_points = 5, jump_size = None, find_valid_jump_size = True):  
   '''
      Draws a star with criscrossing lines.
      
      jump_size determines how many points to skip between connected points.
      an illegal jump size (one that does not result in a valid crisscross star)
      is ignored and replaced with a dot in the center of the star.
   '''
   corners = []
   pi_div_180 = math.pi / 180.0
   half_width = width * 0.5
   half_height = height * 0.5

   degrees = degrees_offset = 90
      
   degrees_incr = 360.0 / num_points

   while degrees < 360.0 + degrees_offset: 
      alpha = degrees * pi_div_180
      sin_alpha = math.sin(alpha);
      cos_alpha = math.cos(alpha);
         
      point_x = (half_width * cos_alpha);
      point_y = (half_height * sin_alpha);

      corners.append((point_x, point_y))
      
      degrees += degrees_incr

   corners.append(corners[0])
      
   if num_points == 6:
      #special case, ignore jump_size
      #rearrange points to draw two polygons
      multiplier = int(num_points / 2)
      
      corners1 = [corners[0], corners[2], corners[4]]
      corners2 = [corners[1], corners[3], corners[5]]
      
      poly1 = Polygon(corners1)
      poly2 = Polygon(corners2)
      return Group([poly1, poly2])

   else:
      if jump_size is None:
         jump_size = int(num_points/2)

      if gcd(num_points, jump_size) != 1:
         if find_valid_jump_size:
            while gcd(num_points, jump_size) != 1:
               jump_size -= 1
         else:
            invalid_star = [[half_width, half_height], [half_width, half_height]]
            return Polygon(invalid_star)
      
      point_order = []
      for i in range(0, num_points):
         point_num =  (i * jump_size) % num_points
         point_order.append(point_num)
      
      corners = [corners[i] for i in point_order]
      
      return  Polygon(corners)


## RUN DEMO CODE
if __name__ == '__main__':
   from chiplotle.tools import io
   from chiplotle.geometry.shapes.star_crisscross import star_crisscross
   from chiplotle.geometry.core.group import Group
   from chiplotle.geometry.transforms.offset import offset
   
   gr1 = Group()
   
   for points in range(5, 26):
      for i in range(1, points):
         s = star_crisscross(1000, 1000, num_points = points, 
            jump_size = i, find_valid_jump_size = False)
         offset(s, ((i - 1) * 1000, -(points - 5) * 1000))
         gr1.append(s)
   
   io.view(gr1)
   





def interpolate_exponential(y1, y2, mu, exp = 1):
   '''Exponential interpolate `y1` and `y2` with `mu` normalized ``[0, 1]``.

   Example::

      >>> mathtools.interpolate_exponential(0, 1, 0.5, 4)
      0.0625

   Set `exp` to the exponent of interpolation.

   '''

   return (y1 * (1 - mu ** exp) + y2 * mu ** exp)




import re

def parse_hpgl_string(arg):
   '''The function takes a string `arg` of HPGL commands, parses them
   (separates them) and returns them in a list.
   '''
   if not isinstance(arg, str):
      raise TypeError('`arg` must be of type string.')

   hpgl_commands = ['AA','AF','AH','AP','AR','AS',
      'BF','BL',
      'CA','CC','CI','CM','CP','CS','CT','CV',
      'DC','DF','DI','DP','DR','DS','DT','DV', #DL
      'EA','EC','EP','ER','ES','EW',
      'FP','FR','FS','FT',
      'GC', #GM GP
      'IM','IN','IP','IV','IW',
      'KY',
      'LB','LO','LT',
      'NR',
      'OA','OC','OD','OE','OF','OG','OH','OI','OK','OL','OO','OP','OS','OT','OW',
      'PA','PB','PD','PG','PM','PR','PS','PT','PU',
      'RA','RO','RR',
      'SA','SC','SI','SL','SM','SP','SR','SS', #SG
      'TL',
      #'UC', UF
      'VS',
      'WD','WG',
      'XT',
      'YT']
   ## TODO: Add all the supported escape (DCI) commands.
   dci_commands = ['\x1b\.\(', '\x1b\.Y']

   # NOTE: added '-' to hpgl_patter because we were barfing on strings like
   # 'PA1.99,-5.00;' -- the ,- construction wasn't being parsed correctly.
   # Don't understand why normal negative integers were being parsed
   # correctly though...
   hpgl_pattern = '[-0-9.,]*|'.join(hpgl_commands)
   dci_pattern = '[0-9.;]*|'.join(dci_commands)
   pattern = hpgl_pattern + "|" + dci_pattern
   ## this assumes that the re will find each and every hpgl command
   ## with the pattern. Any command not matched will effectively be
   ## romoved... we don't want that. 
   result = re.findall(pattern, arg) 
   return result



from chiplotle.core.cfg.cfg import CONFIG_FILE

def read_config_file( ):
   '''Read the content of the config file ``$HOME/.chiplotle/config.py``.
   Returns a dictionary of ``var : value`` entries.'''

   globals = { }
   locals = { }
   execfile(CONFIG_FILE, globals, locals)
   return locals




from chiplotle import *
from chiplotle.hpgl.commands import EA

from py.test import raises

def test_EA_01( ):
   '''EA cannot initialize with a scalar.'''
   assert raises(TypeError, 't = EA(1)')


def test_EA_02( ):
   '''EA cannot initialize with a list of length > 2.'''
   assert raises(ValueError, 't = EA([1, 2, 3, 4])')


def test_EA_03( ):
   '''EA can initialize with a list of length 2.'''
   t = EA([1, 2])
   assert t.xy == Coordinate(1, 2)
   assert t.format == 'EA1,2;'





from chiplotle.geometry.core.group import Group

class Layer(Group):

   def __init__(self, shapes, name):
      Group.__init__(self, shapes = shapes)
      self.name = name

   def __str__(self):
      return 'Layer({0}, {1})'.format(len(self), self.name)



from chiplotle.plotters.drawingplotter import _DrawingPlotter

class HP7585B(_DrawingPlotter):
   '''Use with Houston Instruments DMP-60.'''
   def __init__(self, ser, **kwargs):
      self.allowedHPGLCommands = tuple(['\x1b.', 'AA','AP','AR',
         'AS','BL','CA','CI','CP','CS','CT','CV','DC','DF',
         'DI','DP','DR','DT','EA','EP','ER','ES','EW','FP',
         'FT','GC','GM','GP','IM','IN','IP','IW','KY',
         'LB','LO','LT','NR','OA','OB','OC','OD','OE','OF','OG','OH','OI',
         'OK','OL','OO','OP','OS','OT','OW','PA','PB','PD','PM','PR',
         'PT','PU','RA','RL','RO','RR','SA','SC','SG','SI','SL','SM',
         'SP','SR','SS','TL','UF','VS','WG','XT','YT'])

      _DrawingPlotter.__init__(self, ser, **kwargs)
      self.type = "HP7585B"




from chiplotle import *
from chiplotle.hpgl.commands import *
from chiplotle.tools.hpgltools import relativize

def test_relativize_01( ):
   '''A sequence or PAs is converted into a sequence or PRs.'''
   t = [PA([(1, 1)]), PA([(2, 2)]), PA([(3, 2)])]
   r = relativize(t)

   'r = [PR((1, 1)), PR((1, 0))]'
   assert len(r) == len(t) - 1
   assert isinstance(r[0], PR)
   assert isinstance(r[1], PR)
   assert r[0].xy == CoordinateArray([(1, 1)])
   assert r[1].xy == CoordinateArray([(1, 0)])


def test_relativize_02( ):
   '''Long PAs are converted to long PRs.'''
   t = [PA([(1, 1), (2, 3), (3, 2)])]
   r = relativize(t)

   'r = [PR([(1, 2), (1, -1)])]'
   assert len(r) == 1
   assert isinstance(r[0], PR)
   assert r[0].xy == CoordinateArray([(1, 2), (1, -1)])


def test_relativize_03( ):
   '''Long PAs are converted to long PRs.'''
   t = [PA([(3, 2)]), PA([(1, 1), (2, 3), (3, 2)])]
   r = relativize(t)

   'r = [PR(-2, -1), PR([(1, 2), (1, -1)])]'
   assert len(r) == 2
   assert isinstance(r[0], PR)
   assert isinstance(r[1], PR)
   assert r[0].xy == CoordinateArray([(-2, -1)])
   assert r[1].xy == CoordinateArray([(1, 2), (1, -1)])


def test_relativize_04( ):
   '''PUs and PDs are left alone.'''
   t = [PA([(1, 1)]), PU([(2, 2)]), PU([(3, 2)])]
   r = relativize(t)

   'r = [PU((2, 2)), PU((3, 2))]'
   assert len(r) == 2
   assert isinstance(r[0], PU)
   assert isinstance(r[1], PU)
   assert r[0].xy == CoordinateArray([(2, 2)])
   assert r[1].xy == CoordinateArray([(3, 2)])


def test_relativize_05( ):
   '''PUs and PDs are left alone.'''
   t = [PA([(1, 1)]), PR([(2, 1)]), PU([(2, 2)]), PU([(3, 2)])]
   r = relativize(t)

   'r = [PR((2, 1)), PU((2, 2)), PU((3, 2))]'
   assert len(r) == 3
   assert isinstance(r[0], PR)
   assert isinstance(r[1], PU)
   assert isinstance(r[2], PU)
   assert r[0].xy == CoordinateArray([(2, 1)])
   assert r[1].xy == CoordinateArray([(2, 2)])
   assert r[2].xy == CoordinateArray([(3, 2)])


def test_relativize_06( ):
   '''RAs are converted into RRs.'''
   t = [PR([(1, 2)]), RA((3, 3))]
   r = relativize(t)

   '[PR((1, 2)), RR((2, 1))]'
   assert len(r) == 2
   assert isinstance(r[0], PR)
   assert isinstance(r[1], RR)
   assert r[0].xy == CoordinateArray([(1, 2)])
   assert r[1].xy == Coordinate(2, 1)



from chiplotle import *
from chiplotle.hpgl.commands import PA

def test_PA_01( ):
   t = PA( )

   assert t.xy == CoordinateArray([ ])
   assert isinstance(t.xy, CoordinateArray)
   assert t.format == 'PA;'


## eq ##

def test_PA__eq__01( ):
   assert PA([(1,2),(3,4)]) == PA([(1,2),(3,4)])




from chiplotle.core.cfg.cfg import CONFIG_DIR
from chiplotle.tools.io.export import export
from chiplotle.tools.io._open_file import _open_file
import os
import time

def view(expr, fmt='eps'):
   '''Displays Chiplotle-HPGL objects for prevewing.

   - `expr` can be an iterable (e.g., list) or a Chiplotle-HPGL object.
   - `fmt` is the file format to which the given `expr` will be 
      converted for viewing. The default is 'eps'.
   '''

   outdir   = os.path.join(CONFIG_DIR, 'output')
   filename = time.strftime('%Y%m%d_%H%M%S')
   filepath = os.path.join(outdir, filename)
   imgfile  = export(expr, filepath, fmt)

   ## show!
   _open_file(imgfile)



from chiplotle.geometry.shapes.rectangle import rectangle
from chiplotle.geometry.core.group import Group

def frame(width, height, inset):
   '''A frame (rectangle within a rectangle) with a width, height, and inset.

   - `width` is the width of the frame.
   - `height` is the height of the frame.
   - `inset` is the distance to inset the inner rectangle from the outer.
   '''

   r1 = rectangle(width, height)
   r2 = rectangle(width - (inset * 2), height - (inset * 2))
   return Group([r1, r2])



## RUN DEMO CODE
if __name__ == '__main__':
   from chiplotle.tools import io

   f1 = frame(1000, 500, inset = 20)
   io.view(f1)



from chiplotle.core.cfg.cfg import LOG_FILE
from chiplotle.core.cfg.get_config_value import get_config_value
import logging

def get_logger(name):
   logger = logging.getLogger(name)
   logger.setLevel(logging.DEBUG)
   string = '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
   formatter = logging.Formatter(string)
   fh = logging.FileHandler(LOG_FILE, 'w')
   fh.setLevel(logging.DEBUG)
   fh.setFormatter(formatter)
   logger.addHandler(fh)

   if get_config_value('verbose') == True:
      string = '%(name)s - %(levelname)s - %(message)s'
      formatter = logging.Formatter(string)
      s = logging.StreamHandler( )
      s.setLevel(logging.INFO)
      s.setFormatter(formatter)
      logger.addHandler(s)

   return logger
   



from chiplotle.geometry.core.path import Path

def path(points):
   '''A simple path.'''
   return Path(points)



## DEMO CODE
if __name__ == '__main__':
   from chiplotle.tools import io
   points  = [(0, 0), (10, 10), (-10, 10), (-10, -10), (10, -10), (0, 0)]
   p = path(points)
   io.view(p)



from chiplotle.geometry.core.path import Path

def line(startpoint, endpoint):
   '''Returns a Path with only two points. 
   The path describes a simple straight line.
   '''
   return Path([startpoint, endpoint])


## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   
   l = line((0,0), (1000,1000))
   print l.format
   io.view(l)



from chiplotle.geometry.core.hpglformatvisitor import HPGLFormatVisitor
from chiplotle.geometry.core.metadata import MetaData
from chiplotle.geometry.core.shapepropertiesmixin import _ShapePropertiesMixin

class _Shape(_ShapePropertiesMixin):
   '''Abstract class from which all geometric shapes inherit.'''

   language = 'HPGL'

   def __init__(self):
      self.formatters = set( )
      self.layer = None
      self.meta = MetaData()


   @property
   def format(self):
      if _Shape.language == 'HPGL':
         v = HPGLFormatVisitor()
      else:
         raise ValueError('Sorry, only HPGL supported at the moment.')
      v.visit(self)
      return v.format


   ## overrides ##

   def __repr__(self):
      return str(self)

   def __str__(self):
      name = self.meta.name or ''
      tags = self.meta.tags or ''
      return '%s(%d) %s %s' % (self.__class__.__name__, 
                               len(self), 
                               name, 
                               tags)



from chiplotle import *
from chiplotle.hpgl.commands import *
import math
from py.test import raises

def test_rotate_hpglprimitives_01( ):
   t = [PU( ), PD( ), PA([(1, 1), (-1, 1)]), PR([(1, 1), (-1, 1)]), 
      ER((1, 2)), EA((1, 2)), CI(10)]
   hpgltools.rotate_hpglprimitives(t, math.pi / 2.0)
   assert len(t) == 7
   print t[2].xy
   ## NOTE: why is this not working?
   #assert t[2].xy == CoordinateArray([Coordinate(1, -1), Coordinate(1, 1)])

   



from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.core.path import Path
from chiplotle.tools.mathtools import superformula
import math

def supershape(width, height, m, n1, n2, n3, 
   point_count=100, percentage=1.0, a=1.0, b=1.0, travel=None):
   '''Supershape, generated using the superformula first proposed 
   by Johan Gielis.

   - `points_count` is the total number of points to compute.
   - `travel` is the length of the outline drawn in radians. 
      3.1416 * 2 is a complete cycle.
   '''
   travel = travel or (math.pi * 2)

   ## compute points...
   phis = [i * travel / point_count 
      for i in range(int(point_count * percentage))]
   points = [superformula(a, b, m, n1, n2, n3, x) for x in phis]

   ## scale and transpose...
   path = [ ]
   for x, y in points:
      x *= width
      y *= height
      path.append(Coordinate(x, y))

   return Path(path)


## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   from chiplotle.geometry.core.path import Path
   e = supershape(100, 120, 5, 3.3, 2, 3, b=0.75, travel = 4*math.pi)
   io.view(e)



from chiplotle.tools import mathtools
import py.test


def test_mathtools_factors_01( ):
   assert py.test.raises(TypeError, 'mathtools.factors(7.5)')
   assert py.test.raises(ValueError, 'mathtools.factors(0)')
   

def test_mathtools_factors_02( ):
   t = mathtools.factors(2)
   assert t == [1, 2]
   

def test_mathtools_factors_03( ):
   t = mathtools.factors(3)
   assert t == [1, 3]
   

def test_mathtools_factors_04( ):
   t = mathtools.factors(4)
   assert t == [1, 2, 2]
   

def test_mathtools_factors_05( ):
   t = mathtools.factors(6)
   assert t == [1, 2, 3]
   

def test_mathtools_factors_06( ):
   t = mathtools.factors(12)
   assert t == [1, 2, 2, 3]



from chiplotle.hpgl.abstract.hpgl import _HPGL

def is_primitive_absolute(command):
   '''Returns True of `command` is a primitive HPGL with absolute position,
   False if `command` is a non-absolute position primitive HPGL. 
   Otherwise the function raises a TypeError exception.'''

   if not isinstance(command, _HPGL):
      raise TypeError('command is not an _HPGL')
   if command._name in ('AA', 'EA', 'PA', 'RA'):
      return True
   else:
      return False



from chiplotle.geometry.core.path import Path
from chiplotle.geometry.core.group import Group
from chiplotle.geometry.shapes.arc_ellipse import arc_ellipse

import math

def arc_circle(radius, 
               start_angle, end_angle, 
               segments = 100, segmentation_mode = '2PI'):
   '''
   Constructs an arc from a circle with the given radius,
   and number of segments. Arc goes from start_angle to end_angle,
   both of which are in radians.

      - `segmentation_mode` : '2PI' or 'arc'. The first segments
         the whole circle into the given number of segments,
         the second segments the arc.
   '''
   radius = radius * 2.0 
   return arc_ellipse(radius, radius, 
                      start_angle, end_angle, 
                      segments, segmentation_mode)



## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   
   gr = Group()
   
   for radius in range(100, 1000, 100):
       ac = arc_circle(radius, 1.0, math.pi)
       assert isinstance(ac, Path)
       gr.append(ac)

   io.view(gr, 'png')





def is_flat_list(lst):
   '''Returns True if list is flat, false otherwise.'''
   if isinstance(lst, (list, tuple)):
      for e in lst:
         if isinstance(e, (list, tuple)):
            return False
      else:
         return True
   else:
      return False



import math


def rotate_3d(xyz, xyzrot):
   '''3D rotation. 

   - `xyz` is a triple (x, y, z) of coordinates.
   - `xyzrot` is a triple (xr, yr, zr) of angles of roation.
   '''
   if not len(xyz) == 3:
      raise ValueError('Coordinate tuple `xyz` is not a triple.')
   if not len(xyzrot) == 3:
      raise ValueError('Rotation tuple `xyzrot` is not a triple.')
   x, y, z = xyz
   ## z rotation..
   x = x * math.cos(xyzrot[2]) + y * math.sin(xyzrot[2]) 
   y = - x * math.sin(xyzrot[2]) + y * math.cos(xyzrot[2]) 
   ## y rotation..
   x = x * math.cos(xyzrot[1]) + z * math.sin(xyzrot[1]) 
   z = - x * math.sin(xyzrot[1]) + z * math.cos(xyzrot[1]) 
   ## x rotation..
   y = y * math.cos(xyzrot[0]) + z * math.sin(xyzrot[0]) 
   z = - y * math.sin(xyzrot[0]) + z * math.cos(xyzrot[0]) 
   return (x, y, z)



from chiplotle.geometry.core.coordinate import Coordinate

def scale(coords, value, pivot):
   '''Scales the given coordinates by the given value
   around the given pivot point.
   - `coords` is a CoordinateArray.
   - `value` is a tuple or Coordinate.
   - `pivot` is a tuple or Coordinate.
   '''
   pivot = Coordinate(*pivot)
   try: 
      ## NOTE Coordinate is a bad name for this. 
      ## Vector is a more general notion and thus more appropriate here.
      value = Coordinate(*value)
   except TypeError:
      pass

   ## TODO use matrices to make this more efficient? 
   if pivot == Coordinate(0, 0):
      coords *= value
   else:
      offset_points = coords - pivot
      scaled_points = offset_points * value
      coords = scaled_points + pivot
   return coords




from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.path import Path
import py.test


def test_shapes_group__init__01( ):
   '''A Group can be empty.'''
   t = Group([])
   assert len(t) == 0

def test_shapes_group__init__02( ):
   '''A Group can take no parameters.'''
   t = Group()
   assert len(t) == 0


def test_shapes_group__init__03( ):
   '''A Group can be initialized with a list of Paths.'''
   t = Group([Path([(1, 2), (3, 4)])])
   assert len(t) == 1
   assert t[0] == Path([(1, 2), (3, 4)])


def test_shapes_group__init__04( ):
   '''A Group can be initialized with another Group.'''
   t = Group([Group([ ]), Path([(1, 2), (3, 4)])])
   assert len(t) == 2
   assert t[0] == Group([ ])
   assert t[1] == Path([(1, 2), (3, 4)])


def test_shapes_group__init__05( ):
   '''A Group cannot be initialized with a non-_Shape.'''
   assert py.test.raises(TypeError, 'Group([1, 2, 3])')
   assert py.test.raises(TypeError, 'Group("a")')
   assert py.test.raises(TypeError, 'Group(0)')
  




from chiplotle import *
import math


def test_rotate_2d_args_01( ):
   '''Rotate 2d can take a tuple.'''
   xy = (1, 2)
   t = mathtools.rotate_2d(xy, 0)
   assert t == Coordinate(*xy)


def test_rotate_2d_args_02( ):
   '''Rotate 2d can take a Coordinate.'''
   xy = Coordinate(1, 2)
   t = mathtools.rotate_2d(xy, 0)
   assert t == xy


def test_rotate_2d_args_03( ):
   '''Rotate 2d can take a list of tuples.'''
   xy = [(1, 2), (2, 3)]
   t = mathtools.rotate_2d(xy, 0)
   assert t == CoordinateArray(xy)


def test_rotate_2d_args_04( ):
   '''Rotate 2d can take a CoordinateArray.'''
   xy = CoordinateArray([(1, 2), (2, 3)])
   t = mathtools.rotate_2d(xy, 0)
   assert t == xy


def test_rotate_2d_args_05( ):
   '''Rotate 2d returns a Coordinate if it gets a pair.'''
   xy = Coordinate(1, 2)
   t = mathtools.rotate_2d(xy, 0)
   assert isinstance(t, Coordinate)


def test_rotate_2d_args_05( ):
   '''Rotate 2d returns a CoordinateArray if it gets a list of coordinates.'''
   xy = [(1, 2), (3, 4)]
   t = mathtools.rotate_2d(xy, 0)
   assert isinstance(t, CoordinateArray)


## values ##

def test_rotate_2d_04( ):
   '''Rotate 2d leaves (x, y) unchanged on 0 rotation.'''
   xy = (0, 0)
   t = mathtools.rotate_2d(xy, 0)
   assert t == Coordinate(*xy)
   xy = (1, 0)
   t = mathtools.rotate_2d(xy, 0)
   assert t == Coordinate(*xy)
   xy = (-1, 0)
   t = mathtools.rotate_2d(xy, 0)
   assert t == Coordinate(*xy)
   xy = (-1, 1)
   t = mathtools.rotate_2d(xy, 0)
   assert t == Coordinate(*xy)
   xy = (-1, -2)
   t = mathtools.rotate_2d(xy, 0)
   assert t == Coordinate(*xy)


def test_rotate_2d_05( ):
   '''Rotate 2d turns counter-clockwise.'''
   xy = (1, 0)
   t = mathtools.rotate_2d(xy, math.pi / 2)
   assert round(t[0], 6) == 0
   assert round(t[1], 6) == 1
   t = mathtools.rotate_2d(xy, math.pi / 4)
   assert round(t[0], 3) == 0.707
   assert round(t[1], 3) == 0.707
   xy = (-1, 0)
   t = mathtools.rotate_2d(xy, math.pi / 2)
   assert round(t[0], 6) == 0
   assert round(t[1], 6) == -1
   t = mathtools.rotate_2d(xy, math.pi / 4)
   assert round(t[0], 3) == -0.707
   assert round(t[1], 3) == -0.707



def test_rotate_2d_06( ):
   '''Rotate 2d returns correct values.'''
   xy = (1, 0)
   t = mathtools.rotate_2d(xy, math.pi / 2)
   assert round(t[0], 6) == 0
   assert round(t[1], 6) == 1


## pivot ##

def test_rotate_2d_pivot_01( ):
   '''Rotate 2d can rotate around an arbitrary pivot point.'''
   xy = (1, 0)
   t = mathtools.rotate_2d(xy, math.pi / 1.5, (1, 0))
   assert round(t[0], 6) == 1
   assert round(t[1], 6) == 0


def test_rotate_2d_pivot_02( ):
   '''Rotate 2d can rotate around an arbitrary pivot point.'''
   xy = (1, 0)
   t = mathtools.rotate_2d(xy, math.pi / 2, (1, 1))
   assert round(t[0], 6) == 2
   assert round(t[1], 6) == 1
   t = mathtools.rotate_2d(xy, math.pi / 4, (1, 1))
   r = Coordinate(0.707, -0.707) + Coordinate(1, 1)
   assert round(t[0], 3) == round(r[0], 3)
   assert round(t[1], 3) == round(r[1], 3)





from chiplotle.plotters.drawingplotter import _DrawingPlotter

class HP7576A(_DrawingPlotter):
   def __init__(self, ser, **kwargs):
      ## allowedHPGLCommands must be set prior to base class init.
      self.allowedHPGLCommands = tuple(['\x1b.','AA','AP','AR','CA','CI',
         'CM','CP','CS','CT','DC','DF','DI','DP','DR','DS','DT','DV','EA',
         'EP','ER','ES','EW','FP','FT','IM','IN','IP','IV','IW','LB','LO',
         'LT','NR','OA','OC','OD','OE','OF','OH','OI','OO','OP','OS','OT',
         'OW','PA','PD','PM','PR','PS','PT','PU','RA','RO','RR','SA','SC',
         'SI','SL','SM','SP','SR','SS','TL','UC','VS','WG','XT','YT'])

      _DrawingPlotter.__init__(self, ser, **kwargs)
      self.type = "HP7576A"




from chiplotle.geometry.core.coordinate import Coordinate

class _PlotterMargins(object):
   def __init__(self, plotter, queryCommand):  
      self._plotter = plotter
      self._queryCommand = queryCommand


   ## PROPERTIES ##


   @property
   def left(self):
      return self.all_coordinates[0]
      
   @property
   def bottom(self):
      return self.all_coordinates[1]

   @property
   def right(self):
      return self.all_coordinates[2]
      
   @property
   def top(self):
      return self.all_coordinates[3]

   @property
   def width(self):
      x1, y1, x2, y2 = self.all_coordinates
      return x2 - x1

   @property
   def height(self):
      x1, y1, x2, y2 = self.all_coordinates
      return y2 - y1
   
   @property
   def center(self):
      #return (self.right + self.left) / 2., (self.top + self.bottom) / 2.
      x = (self.right + self.left) / 2.
      y = (self.top + self.bottom) / 2.
      return Coordinate(x, y)

   @property
   def bottom_left(self):
      coords = self.all_coordinates
      return Coordinate(*coords[0:2])

   @property
   def bottom_right(self):
      coords = self.all_coordinates
      return Coordinate(coords[2], coords[1])

   @property
   def top_right(self):
      coords = self.all_coordinates
      return Coordinate(*coords[2:4])

   @property
   def top_left(self):
      coords = self.all_coordinates
      return Coordinate(coords[0], coords[3])

   @property
   def all_coordinates(self):
      self._plotter._serial_port.flushInput( )
      self._plotter._write_string_to_port(self._queryCommand.format)
      m = self._plotter._read_port( ).rstrip('\r').split(',')
      return tuple([eval(n) for n in m])
      

   ## METHODS ##

   def draw_outline(self, pen=1):
      pen = self._plotter._hpgl.SP(pen)
      pa = self._plotter._hpgl.PA([self.bottom_left])
      rec = self._plotter._hpgl.EA(self.top_right)
      self._plotter.write([pen, pa, rec])

   def draw_corners(self, pen=1):
      from chiplotle.hpgl.compound import Cross
      coords = self.all_coordinates
      size = 100
      corners = [ ]
      corners.append(Cross(coords[0:2], width = size, height = size, pen = 1))
      corners.append(Cross((coords[0], coords[3]), size,  size))
      corners.append(Cross((coords[2], coords[1]), size,  size))
      corners.append(Cross(coords[2:4], size,  size))
      self._plotter.write(corners)


   ## OVERRIDES ##

   def __repr__(self):
      return '%s%s' % (self.__class__.__name__, self.all_coordinates)




from chiplotle.core.interfaces.interface import _Interface
from chiplotle.plotters.margins.marginssoft import MarginsSoft
from chiplotle.plotters.margins.marginshard import MarginsHard

class MarginsInterface(_Interface):
   
   def __init__(self, client):
      _Interface.__init__(self, client)
      self._soft = MarginsSoft(client)
      self._hard = None
      ## check if hard margin (OH) is supported by plotter...
      if 'OH' in client.allowedHPGLCommands:
         self._hard = MarginsHard(client)


   @property
   def soft(self):
      '''Read-only reference to MarginsSoft.'''
      return self._soft

   @property
   def hard(self):
      '''Read-only reference to MarginsHard.'''
      return self._hard



from chiplotle.tools.logtools.get_logger import get_logger

def apply_logger(f):
   '''Applies a logger object to the 'wrapped' function.'''
   logger = get_logger(f.func_name)
   f.logger = logger
   return f




from chiplotle.tools.mathtools.difference import difference
from chiplotle.geometry.core.path import Path
import math

def perpendicular_displace(path, displacements):
   '''Displaces a path along its perpendiculars.
   
   - `path` is a Path instance.
   - `displacement` is a list of displacement values (scalars). 

   Precondition: len(path) == len(displacement)
   ''' 
   if not isinstance(path, Path):
      raise TypeError('`path` should be of type Path.')
   if len(path.points) != len(displacements):
      raise ValueError('len(path) != len(displacements).')

   ## TODO may want to check here for identical consecutive coords
   ## to avoid 0 differences.
   result = [ ]
   d_points = difference(path.points)
   d_points.append(d_points[-1])
   for i in range(len(path.points)):
      perp = d_points[i].perpendicular.normalized
      disp = perp * displacements[i]
#      perp = d_points[i].perpendicular
#      mag = perp.magnitude
#      if mag == 0: ## two points are overlapping.
#         disp = 0
#      else:
#         disp = perp / mag * displacements[i]

      result.append(path.points[i] + disp)
   path.points = Path(result).points


## ~~~~~~~~~~~~~~~ demo ~~~~~~~~~~~
if __name__ == '__main__':
   from chiplotle import *
   from chiplotle.hpgl.formatters.pen import Pen
   import copy
   from random import randint
   import math

   rndpath = [(randint(0, 2000), randint(0, 2000)) for i in range(2 * 4)]
   p = bezier_path(rndpath, .1, 80)
   print '*** ', p
   pc = copy.deepcopy(p)
   Pen(2)(pc)

   d = [math.sin(x / math.pi) * 30 for x in range(len(p.points))]

   perpendicular_displace(p, d)
   io.view(group([p, pc]))



from chiplotle.hpgl.abstract.hpglprimitive import _HPGLPrimitive
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.tools.hpgltools.convert_relatives_to_absolutes import \
   convert_relatives_to_absolutes


def get_all_coordinates(arg):
   '''Returns all absolute coordinates for a given list of Chiplotle-HPGL commands.
   
   Example::

      >>> t = [PA((1, 2)), PR((1, 1)), ER((1, 1)), CI(100)]
      >>> c = hpgltools.get_all_coordinates(t)
      >>> print c
      [CP(1, 2), CP(2, 3), CP(3, 4)]
   '''
   if not isinstance(arg, (list, tuple)):
      raise TypeError('`arg` must be list or tuple')

   arg = convert_relatives_to_absolutes(arg)

   result = [ ]
   for e in arg:
      if isinstance(e, _HPGLPrimitive) and hasattr(e, 'xy'):
         if isinstance(e.xy, CoordinateArray):
            result.extend(e.xy)
         else:
            result.append(e.xy)
      else:
         print '"%s" has not abs coords.' % e
   return result



from chiplotle.hpgl.commands import ES

def test_ES_01( ):
   t = ES( )
   assert t.charspace == None
   assert t.linespace == None
   assert t.format == 'ES;'
   

def test_ES_02( ):
   t = ES(0)
   assert t.charspace == 0
   assert t.linespace == None
   assert t.format == 'ES0.00;'

def test_ES_03( ):
   t = ES(1)
   assert t.charspace == 1
   assert t.linespace == None
   assert t.format == 'ES1.00;'

def test_ES_04( ):
   t = ES(1, 0)
   assert t.charspace == 1
   assert t.linespace == 0
   assert t.format == 'ES1.00,0.00;'

def test_ES_04( ):
   t = ES(1, 1)
   assert t.charspace == 1
   assert t.linespace == 1
   assert t.format == 'ES1.00,1.00;'




from chiplotle.hpgl.abstract.hpgl import _HPGL

def _scale_command(obj, val):
   attrs = obj.__dict__.keys()
   for scalable_attr in obj.__class__._scalable:
      a = getattr(obj, scalable_attr)
      setattr(obj, scalable_attr, a * val)

def scale(obj, val):
   if isinstance(obj, _HPGL):
      _scale_command(obj, val)
   if isinstance(obj, (list, tuple)):
      for c in obj:
         _scale_command(c, val)



from chiplotle.hpgl.commands import SI
import py.test

def test_SI_01( ):
   '''SI has default values.'''
   t = SI( )
   
   assert t.format == 'SI;'


def test_SI_02( ):
   t = SI(3, 2)
   
   assert t.format == 'SI3.00,2.00;'


def test_SI_03( ):

   t = SI(3)

   assert py.test.raises(Warning, 't.format')



from chiplotle.hpgl import commands 
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.tools.hpgltools.inflate_hpgl_string import inflate_hpgl_string
from sys import maxint

class VirtualSerialPort():
   def __init__(self, left_bottom, right_top):
      left_bottom = Coordinate(*left_bottom)
      right_top = Coordinate(*right_top)
      #print "I am a virtual serial port!"
      self._received_commands_string = ""
      self._next_query_value = ''
      self.commanded_x = 0
      self.commanded_y = 0
      #pen_status: 0 == up, 1 == down
      self.pen_status = 0
      self.left = left_bottom.x
      self.right = right_top.x
      self.bottom = left_bottom.y
      self.top = right_top.y
      self.buffer_size = maxint
      self.portstr = "VirtualSerialPort"
      
   def write(self, command):
      '''
      we assume that commands are sent either as single escaped commands,
      like B() or On(), or as single or strings of regular text commands,
      like PA0,0;PD;
      
      if an escape command is somehow inserted into a string of text commands
      we will not catch it! but that shouldn't happen...
      
      '''
      #print "vsp got command: " + command
     
      '''this seems dumb, but don't know how else to do it.
         gotta iterate through all possible weirdo commands.
      '''
      
      #make sure we received a string, not a tuple or something
      assert type(command) is str
            
      if command.startswith(commands.B().format):
         #let's say we have 4MB of memory to avoid buffered writes
         self._next_query_value = self.buffer_size
         return
      elif command.startswith(commands.On().format):
         #print "vsp: ignoring On() command."
         return
      elif command.startswith(commands.OH().format):
         #hard margins
         out_string = "%d, %d, %d, %d\r" % (self.left, self.bottom, self.right, self.top)
         self._next_query_value = out_string
         return
      elif command.startswith(commands.OW().format):
         #soft margins
         out_string = "%d, %d, %d, %d\r" % (self.left, self.bottom, self.right, self.top)
         self._next_query_value = out_string
         return
      elif command.startswith(commands.OI().format):
         self._next_query_value = "VirtualPlotter\r"
         return
      elif command.startswith(commands.OA().format):
         #actual position
         out_string = "%i, %i, %i\r" % (self.commanded_x, self.commanded_y, self.pen_status)
         self._next_query_value = out_string
         return
      elif command.startswith(commands.OC().format):
         #commanded position
         out_string = "%i, %i, %i\r" % (self.commanded_x, self.commanded_y, self.pen_status)
         self._next_query_value = out_string
         return
      elif command.startswith(commands.OP().format):
         #output P1P2
         out_string = "%d, %d, %d, %d\r" % (self.left, self.bottom, self.right, self.top)
         self._next_query_value = out_string
         return

      #if we made it here then we're normal HPGL
      
      self._received_commands_string += command
         
      #store commanded position data
      #this breaks for buffered writes since we don't always
      #receive a full PA1000,1000 type command
      
      split_data = command.split(';')
      
      for point in split_data:
         if point.startswith("PA"):
            point_parts = point.strip("PA").split(',')
            self.commanded_x = eval(point_parts[len(point_parts) - 2])
            self.commanded_y = eval(point_parts[len(point_parts) - 1])
         elif point.startswith("PD"):
            if ',' in point:
               point_parts = point.strip("PD").split(',')
               self.commanded_x = eval(point_parts[len(point_parts) - 2])
               self.commanded_y = eval(point_parts[len(point_parts) - 1])
            self.pen_status = 1
         elif point.startswith("PU"):
            if ',' in point:
               point_parts = point.strip("PU").split(',')
               self.commanded_x = eval(point_parts[len(point_parts) - 2])
               self.commanded_y = eval(point_parts[len(point_parts) - 1])
            self.pen_status = 0
         if point.startswith("PR"):
            point_parts = point.strip("PR").split(',')
            self.commanded_x += eval(point_parts[len(point_parts) - 2])
            self.commanded_y += eval(point_parts[len(point_parts) - 1])   

      #print "commanded_x: %i commanded_y: %i" % (self.commanded_x, self.commanded_y)
         
   def flush(self):
      pass
      
   def flushInput(self):
      #print "vsp: flushed input."
      pass
      
   def flushOutput(self):
      #print "vsp: flushed output."
      pass
   
   def clear(self):
      #this method doesn't exist for real serial ports
      #we use it to erase the stored commands so that you can reset
      #a virtual plotter to a blank state
      
      self._received_commands_string = ""
      
   #what's a reasonable value here?
   def inWaiting(self):
      return 10
      
      
   def readline(self, eol=None):
      #print "returning: " + self._next_query_value
      return_value = self._next_query_value
      self._next_query_value = None
      return return_value
      
      
   def get_received_commands(self):
      return inflate_hpgl_string(self._received_commands_string)

   def get_received_commands_string(self):
      return self._received_commands_string
      
   @property
   def format(self):
      '''This is so that a VirtualPlotter can call serial_port.format to retrieve
      stored commands and send them to io.view()
      '''
      return self._received_commands_string



from chiplotle.geometry.core.polygon import Polygon
from chiplotle.geometry.transforms.scale import scale
import math

def circle(radius, segments = 36):
   '''Returns a circle.'''
   coords = [(math.cos(math.pi * 2 / segments * r) * radius, 
              math.sin(math.pi * 2 / segments * r) * radius) 
              for r in range(segments)]
   return Polygon(coords)


## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   e = circle(1000)
   io.view(e)



from chiplotle import *
from py.test import raises

def test_coordinatearray_append_01( ):
   '''CoordinateArray cannot be appended an (x, y) pair.'''
   t = CoordinateArray( )
   assert raises(TypeError, 't.append((0, 0))')


def test_coordinatearray_append_02( ):
   '''CoordinateArray can be appended an Coordinate.'''
   t = CoordinateArray( )
   t.append(Coordinate(0, 0))
   assert len(t) == 1
   assert isinstance(t[0], Coordinate)





from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.hpgl.commands import SC, IP, IW

'''
Interactive plotter routines.

Note that these routines may not work on your plotter. They are
known to work on most HP and Roland plotters, but they do NOT
work on the Houston Instruments DMP-60 (and probably other Houston
Instruments plotters). This is because the DMP-60 interprets manual 
pen moves as attempts to reset the origin, which interferes with 
these routines.

'''

def interactive_set_plot_window(plotter):
   '''
   Interactive routine to manually move the pen to set the margins of the plotting window.
   '''
   
   plotter.write(IP())
   plotter.write(IW())
   #plotter.write(SC())
   
   print "Interactive set plot window:"
   print "Move pen to lower left and press enter."
   raw_input()
   position = plotter.actual_position[0]
   x1 = position.x
   y1 = position.y
   
   print "Move pen to upper right and press enter."
   raw_input()
   position = plotter.actual_position[0]
   x2 = position.x
   y2 = position.y
   
   plotter.set_plot_window(Coordinate(x1, y1), Coordinate(x2, y2))

   '''
from chiplotle.plotters.interactive.interactive_commands import *
interactive_set_plot_window(plotter)
   '''
   
   

def interactive_set_plot_window_and_units(plotter):
   '''
   User sets window size and then defines units inside of that window.
   '''
   plotter.write(IP())
   plotter.write(IW())
   plotter.write(SC())

   interactive_set_plot_window(plotter)
   
   print "Enter value for left side (typically 0):"
   left = int(raw_input())
   print "Enter value for bottom side (typically 0):"
   bottom = int(raw_input())
   print "Enter value for right side (width of plot in your units):"
   right = int(raw_input())
   print "Enter value for top side (height of plot in your units):"
   top = int(raw_input())
   
   plotter.write(SC([left,right,bottom,top]))
   print "new soft margins:"
   print plotter.margins.soft

   '''
from chiplotle.interactive.interactive_commands import *
interactive_set_plot_window_and_units(plotter)
   '''
   

def interactive_set_plot_window_auto_units(plotter):
   '''
   User sets window size and then units are automatically set
   to user's choise of inches, millimeters, or centimeters.
   '''
   plotter.write(IP())
   plotter.write(IW())
   plotter.write(SC())
   
   interactive_set_plot_window(plotter)
   
   orig_right = plotter.margins.soft.right
   orig_top = plotter.margins.soft.top
   
   width = plotter.margins.soft.width
   height = plotter.margins.soft.height
   
   inches_w = width/1016.0
   inches_h = height/1016.0
   
   cm_w = width/400.0
   cm_h = height/400.0
   
   mm_w = width/40.0
   mm_h = width/40.0
   
   print "Window size is:"
   print "%f inches x %f inches" % (inches_w, inches_h)
   print "%f cm x %f cm" % (cm_w, cm_h)
   print "%f mm x %f mm" % (mm_w, mm_h)
   
   print "Choose units:"
   print "1) inches"
   print "2) cm"
   print "3) mm"
   
   units = int(raw_input())

   left = plotter.margins.soft.left
   bottom = plotter.margins.soft.bottom
   
   if units == 1:
      right = left + 1016
      top = bottom + 1016
   elif units == 2:
      right = left + 400
      top = bottom + 400
   elif units == 3:
      right = left + 40
      top = righ + 40
   else:
      print "That wasn't one of the choices!"
      return

   plotter.write(IP([left, bottom,right,top]))
   plotter.write(SC([0,1,0,1]))
   #plotter.write(IP([left, bottom, orig_right, orig_top]))
   
   
   #These margins will be WRONG!!! They'll be the floor integer margins,
   #not the margins set via the set_plot_window() above. ARRRRG!
   print "new soft margins:"
   print plotter.margins.soft

   '''
from chiplotle.plotters.interactive.interactive_commands import *
interactive_set_plot_window_auto_units(plotter)
   '''
   

def interactive_define_polygon_simple(plotter):
   '''
   Interactive routine to define points in a PolygonSimple object.
   '''
   from chiplotle.hpgl.compound.polygon_simple import PolygonSimple
   
   points = []

   print "Interactive define PolygonSimple:"
   print "Move pen to each point and press enter. Press x when finished adding points."
   print "The final point (a duplicate of first point) will be added automatically."
   while True:
      input = raw_input()
      if input is 'x':
         break;

      point = Coordinate(plotter.actual_position[0].x, plotter.actual_position[0].y)
      points.append(point)
      print "added:"
      print point
      
   poly = PolygonSimple([0,0], points)
      
   return poly


def interactive_define_rectangle(plotter):
   '''
   Interactive routine to define points in a Rectangle object.
   '''
   from chiplotle.hpgl.compound.rectangle import Rectangle
   
   points = []

   print "Interactive define Rectangle:"
   print "Move pen to lower, left corner and press enter."

   input = raw_input()
   lower_left = Coordinate(plotter.actual_position[0].x, plotter.actual_position[0].y)
   print "lower_left:"
   print lower_left
   
   print "Move pen to upper, right corner and press enter."
   input = raw_input()
   upper_right = Coordinate(plotter.actual_position[0].x, plotter.actual_position[0].y)
   print "upper_right:"
   print upper_right   
      
   rectangle = Rectangle([lower_left], upper_right.x, upper_right.y)
      
   return rectangle

'''
from chiplotle.plotters.interactive.interactive_commands import *
interactive_define_rectangle(plotter)
'''


def interactive_define_ellipse(plotter):
   '''
   Interactive routine to define center and radii of an ellipse.
   '''
   from chiplotle.hpgl.compound.rectangle import Ellipse
   
   points = []

   print "Interactive define Rectangle:"
   print "Move pen to lower, left corner and press enter."

   input = raw_input()
   lower_left = Coordinate(plotter.actual_position[0].x, plotter.actual_position[0].y)
   print "lower_left:"
   print lower_left
   
   print "Move pen to upper, right corner and press enter."
   input = raw_input()
   upper_right = Coordinate(plotter.actual_position[0].x, plotter.actual_position[0].y)
   print "upper_right:"
   print upper_right   
      
   rectangle = Rectangle([lower_left], upper_right.x, upper_right.y)
      
   return rectangle

'''
from chiplotle.plotters.interactive.interactive_commands import *
interactive_define_rectangle(plotter)
'''




#from chiplotle import *
#from chiplotle.core.errors import OperandError
#import py.test
#
#
### ARGUMENT CHECK ##
#
#def test_get_centroid_arg_01( ):
#   '''get_centroid( ) can take a list of Coordinates.'''
#   c = [Coordinate(1, 2), Coordinate(-1, -2)]
#   t = geometrytools.get_centroid(c)
#   assert isinstance(t, Coordinate)
#   assert t == Coordinate(0, 0)
#
#
#def test_get_centroid_arg_02( ):
#   '''get_centroid( ) can take a CoordinateArray.'''
#   c = CoordinateArray([Coordinate(1, 2), Coordinate(-1, -2)])
#   t = geometrytools.get_centroid(c)
#   assert isinstance(t, Coordinate)
#   assert t == Coordinate(0, 0)
#
#
#def test_get_centroid_arg_03( ):
#   '''get_centroid( ) can not take a list of tuples.'''
#   c = [(1, 2), (-1, -2)]
#   assert py.test.raises(TypeError, 't = geometrytools.get_centroid(c)')
#
#
#
### OPERATION ##
#
#def test_get_centroid_01( ):
#   g = [Coordinate(0, 0), Coordinate(10, 10), 
#      Coordinate(0, 10), Coordinate(10, 0)]
#   t = geometrytools.get_centroid(g)
#   assert t == Coordinate(5, 5)
#



from chiplotle.geometry.transforms.offset import offset
from chiplotle.geometry.core.coordinate import Coordinate

def center_at(shape, coord):
   '''Centers shape at the given coordinate.'''
   offset(shape, -shape.center + Coordinate(*coord))
   



from chiplotle import *
import os


def test_01():
   circ     = shapes.circle(100)
   filename = 'circle.hpgl'
   io.save_hpgl(circ, filename)

   assert os.path.exists(filename)

   os.remove(filename)



import operator
import math


class Coordinate(object):

   __slots__ = ('_coords')

#   def __new__(cls, *args):
#      if len(args) == 1 and isinstance(args[0], Coordinate):
#         return args[0]
#      else:
#         return super(Coordinate, cls).__new__(cls)

   def __init__(self, *args):
      if args and not isinstance(args[0], (int, float, long)):
         raise TypeError('Arguments must all be scalars')
      self._coords = list(args)


   @property
   def xy(self):
      return self._coords[0:2]

   @property
   def x(self):
      return self.xy[0]

   @property
   def y(self):
      return self.xy[1]

   @property
   def magnitude(self):
      '''The norm.'''
      return math.sqrt(sum([c**2 for c in self._coords]))

   @property
   def normalized(self):
      return self / self.magnitude

   ## TODO: make multidimensional.
   @property
   def polar(self):
      from chiplotle.tools.mathtools.xy_to_polar import xy_to_polar
      return xy_to_polar(self.xy)

   @property
   def angle(self):
      return self.polar[1]

   ## TODO: make multidimensional.
   @property
   def perpendicular(self):
      return Coordinate(-self.y, self.x)

   ## OVERRIDES ##

   def __getitem__(self, arg):
      return self._coords[arg]

   def __iter__(self):
      for e in self._coords:
         yield e

   def __hash__(self):
      return hash(tuple(self._coords))

   def __len__(self):
      return len(self._coords)

   def __repr__(self):
      return 'Coordinate(%s)' % self._coords

   def __str__(self):
      return '<%s>' % ','.join([str(c) for c in self._coords])

   ## math operators ##

   def __abs__(self):
      return Coordinate(*map(abs, self._coords))

   def __add__(self, arg):
      if isinstance(arg, Coordinate):
         coords = map(operator.add, self._coords, arg._coords)
         return Coordinate(*coords)
      try:
         return arg.__radd__(self)
      except Exception:
         raise TypeError
       
#   def __radd__(self, arg):
#      return self + arg

   def __sub__(self, arg):
      if isinstance(arg, Coordinate):
         coords = map(operator.sub, self._coords, arg._coords)
         return Coordinate(*coords)
      try:
         return arg.__rsub__(self)
      except Exception:
         raise TypeError

   def __rsub__(self, arg):
      return - (self - arg)

   def __truediv__(self, arg):
      coords = [c / float(arg) for c in self._coords]
      return Coordinate(*coords)

   def __div__(self, arg):
      return self.__truediv__(arg)

   def __floordiv__(self, arg):
      coords = self.__div__(arg)._coords
      coords = [int(math.floor(c)) for c in coords]
      return Coordinate(*coords)

   def __mul__(self, arg):
      if not isinstance(arg, (Coordinate, int, float, long)):
         raise TypeError
      if isinstance(arg, Coordinate):
         coords = map(operator.mul, self._coords, arg._coords)
      else:
         coords = [c * arg for c in self._coords]
      return Coordinate(*coords)

   def __rmul__(self, arg):
      return self * arg

   def __eq__(self, arg):
      try:
         return self._coords == arg._coords
      except Exception:
         return False

   def __ne__(self, arg):
      return not (self == arg)

   def __neg__(self):
      coords = [-c for c in self._coords]
      return Coordinate(*coords)

   def __invert__(self):
      '''Returns the perpendicular of self.
      http://mathworld.wolfram.com/PerpendicularCoordinate.html
      '''
      return self.perpendicular



if __name__ == '__main__':

   c = Coordinate(1, 2, 3)
   cp = c + c
   cs = c - c
   cm = c * 2
   cd = c / 2
   cn = -c
   print 'len(c) =', len(c)
   print c
   print cp
   print cs
   print cm
   print cd
   print cn

#from __future__ import division
#from chiplotle.core import errors
#import math
#
#class Coordinate(object):
#
#   __slots__ = ('_x', '_y')
#
##   def __new__(cls, *args):
##      if len(args) == 1 and isinstance(args[0], Coordinate):
##         return args[0]
##      else:
##         ## NOTE: Python 2.6.6... DeprecationWarning: object.__new__() takes no parameters
##         #return super(Coordinate, cls).__new__(cls, *args)
##         return super(Coordinate, cls).__new__(cls)
#
#   def __init__(self, *args):
#      if len(args) == 1:
#         if isinstance(args[0], Coordinate):
#            self._x, self._y = args[0].x, args[0].y
##         elif isinstance(args[0], (list, tuple)):
##            self.__init__(*args[0])
#         else:
#            #raise TypeError('argument not recognized by Coordinate.')
#            raise errors.InitParameterError( )
#      elif len(args) == 2:
#         from chiplotle.tools.iterabletools.isiterable import isiterable
##         if isiterable(args[0]) or isiterable(args[1]):
#         if not isinstance(args[0], (float, int, long)) or \
#            not isinstance(args[1], (float, int, long)):
#            raise errors.InitParameterError( )
#         self._x = args[0]
#         self._y = args[1]
#      else:
#         #raise ValueError('too many arguments Coordinate.')
#         raise errors.InitParameterError( )
#
#
#   ## PUBLIC PROPERTIES ##
#
#   @property
#   def xy(self):
#      return self.x, self.y
#
#   @property
#   def x(self):
#      return self._x
#
#   @property
#   def y(self):
#      return self._y
#
#   @property
#   def magnitude(self):
#      '''The norm.'''
#      #return math.sqrt(self.x ** 2 + self.y ** 2)
#      return self.polar[0]
#
#   @property
#   def angle(self):
#      return self.polar[1]
#
#
#   @property
#   def polar(self):
#      from chiplotle.tools.mathtools.xy_to_polar import xy_to_polar
#      return xy_to_polar(self.xy)
#
#   @property
#   def normalized(self):
#      return self / self.magnitude
#
#   @property
#   def perpendicular(self):
#      return Coordinate(-self.y, self.x)
#
#   ## OVERRIDES ##
#
#   def __getitem__(self, arg):
#      return self.xy[arg]
#
#   def __iter__(self):
#      for e in self.xy:
#         yield e
#
#   def __hash__(self):
#      ## TODO: what is the best way to hash this pair?
#      return hash(('CP', self.x, self.y))
#
#   def __len__(self):
#      return 2
#
#   def __repr__(self):
#      return 'Coordinate(%s, %s)' % (self.x, self.y)
#
#   def __str__(self):
#      if isinstance(self.x, int) and isinstance(self.y, int):
#         return '<%i,%i>' % (self.x, self.y)
#      else:
#         return '<%.2f,%.2f>' % (self.x, self.y)
#
#
#   ## math operators ##
#
#   ## addition ##
#
#   def __abs__(self):
#      return Coordinate(abs(self.x), abs(self.y))
#
#   def __add__(self, arg):
#      try:
#         #arg = Coordinate(arg)
#         return Coordinate(self.x + arg.x, self.y + arg.y)
#      except:
#         try:
#            return Coordinate(self.x + arg, self.y + arg)
#         except:
#            try:
#               return arg.__radd__(self)
#            except:
#               raise errors.OperandError
#
#   def __radd__(self, arg):
#      return self + arg
#
#
#   ## substraction ##
#
#   def __sub__(self, arg):
#      try:
#         #arg = Coordinate(arg)
#         return Coordinate(self.x - arg.x, self.y - arg.y)
#      except:
#         try:
#            return Coordinate(self.x - arg, self.y - arg)
#         except:
#            try:
#               return arg.__rsub__(self)
#            except:
#               raise errors.OperandError
#
#   def __rsub__(self, arg):
#      return -(self - arg)
#
#
#   ## division ##
#
#   def __div__(self, arg):
#      return self.__truediv__(arg)
#   
#   def __floordiv__(self, arg):
#      result = self.__div__(arg)
#      return Coordinate(int(math.floor(result.x)), int(math.floor(result.y)))
#
#   def __truediv__(self, arg):
#      try:
#         return Coordinate(self.x / arg.x, self.y / arg.y)
#      except AttributeError:
#         try:
#            return Coordinate(self.x / arg, self.y / arg)
#         except (TypeError, errors.InitParameterError):
#            raise errors.OperandError( )
#
#
#   ## mul ##
#
#   def __mul__(self, arg):
#      try:
#         #coord = Coordinate(arg)
#         return Coordinate(self.x * arg.x, self.y * arg.y)
#      except AttributeError:
#         try:
#            return Coordinate(self.x * arg, self.y * arg)
#         except errors.InitParameterError:
#            raise errors.OperandError( )
#
#
#   def __rmul__(self, arg):
#      return self * arg
#
#   ## ## 
#
#   def __eq__(self, arg):
#      try:
#         #arg = Coordinate(arg)
#         return (self.x == arg.x) and (self.y == arg.y)
#      except AttributeError:
#         return False
#
#   def __invert__(self):
#      '''Returns the perpendicular of self.
#      http://mathworld.wolfram.com/PerpendicularCoordinate.html
#      '''
#      return self.perpendicular
#
#   def __ne__(self, arg):
#      return not (self == arg)
#
#   def __neg__(self):
#      return Coordinate(-self.x, -self.y)
#



from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.geometry.transforms.transformvisitor import TransformVisitor

def offset(shape, value):
   '''In place offsetting.

   - `shape` is the shape to be rotated.
   - `value` is the offset value. Can be a scalar or an (x, y) coordinate.
   '''
   if isinstance(value, (list, tuple)):
      value = Coordinate(*value)

   def offset(coords, value):
      return coords + value

   t = TransformVisitor(offset)
   t.visit(shape, value)


## RUN DEMO CODE
if __name__ == "__main__":
   from chiplotle.geometry.shapes.rectangle import rectangle
   from chiplotle.tools import io
   r0 = rectangle(1000, 400)
   r1 = rectangle(1000, 400)
   r2 = rectangle(1000, 400)
   offset(r1, (0, 1500))
   offset(r2, (100, 200))
   io.view(Group([r0, r1, r2]))




from chiplotle import *
from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.path import Path
from py.test import raises 

'''A Group is offset with the + operator.'''

'''A Group with paths works as expected.'''

def test_shapes_group_add_01( ):
   '''Group + int is not allowed.'''
   a = Group([Path([(1, 2), (3, 4)]), Path([(5, 6), (7, 8)])])
   assert raises(TypeError, 't = a + 2')


def test_shapes_group_add_02( ):
   '''int + Group is not allowed.'''
   a = Group([Path([(1, 2), (3, 4)]), Path([(5, 6), (7, 8)])])
   assert raises(TypeError, 't = 2 + a')



from chiplotle import *
from chiplotle.hpgl.commands import *

def test_convert_relatives_to_absolutes_01( ):
   g = [PA([(1, 2)]), PR([(1, 1), (1, 1)])]
   t = hpgltools.convert_relatives_to_absolutes(g)
   assert len(t) == 2
   assert t[0] == PA([(1, 2)])
   assert t[1] == PA([(2, 3), (3, 4)])


def test_convert_relatives_to_absolutes_02( ):
   '''Initial absolute position is set to (0, 0) when absolute commands are absent before PR.'''
   g = [PR([(1, 1), (1, 1)])]
   t = hpgltools.convert_relatives_to_absolutes(g)
   assert len(t) == 1
   assert t[0] == PA([(1, 1), (2, 2)])


def test_convert_relatives_to_absolutes_03( ):
   '''Edges and archs work.'''
   g = [ER((1, 1)), RR((1, 1)), AR((1, 1), 40), EA((0, 0))]
   t = hpgltools.convert_relatives_to_absolutes(g)
   assert len(t) == 4
   assert t[0] == EA((1, 1))
   assert t[1] == RA((2, 2))
   assert t[2] == AA((3, 3), 40)
   assert t[3] == EA((0, 0))




from chiplotle import *
from chiplotle.hpgl.commands import *

def test_get_all_coordinates_01( ):
   '''get_all_coordinates( ) works with _HPGLPrimitives.'''
   t = [PA([(1, 2)]), PR([(1, 1)]), ER((1, 1)), CI(100)]
   c = hpgltools.get_all_coordinates(t)
   assert len(c) == 3
   assert c[0] == Coordinate(1, 2)
   assert c[1] == Coordinate(2, 3)
   assert c[2] == Coordinate(3, 4)
 




from chiplotle.hpgl.commands import FT
from chiplotle.core.interfaces.formatdecorator import FormatDecorator


class FillType(FormatDecorator):
   
   __doc__ = FT.__doc__

   def __init__(self, filltype=None, space=None, angle=None):
      FormatDecorator.__init__(self)
      self.filltype = filltype
      self.space = space
      self.angle = angle


   @property
   def _subcommands(self):
      return [FT(self.filltype, self.space, self.angle)]



from chiplotle.geometry.core.path import Path
from chiplotle.geometry.transforms.offset import offset

def arrange_shapes_on_path(shapes, path):

   if not isinstance(path, Path):
      raise TypeError
   if len(shapes) != len(path):
      raise ValueError('len(shapes) == len(path) must be true.')

   for shape, coord in zip(shapes, path.points):
      offset(shape, coord)



if __name__ == '__main__':
   from chiplotle import *
   import random

   count = 10
   coords = [random.randint(0, 4000) for i in range(count * 2)]
   circs = [circle(100) for i in range(count)]
   p = Path(coords)
   PenDecorator(Pen(2))(p)

   arrange_shapes_on_path(circs, p)

   io.view(group(circs + [p]))



from chiplotle.core.visitor import Visitor

class TransformVisitor(Visitor):
   '''"Crawler" pattern encapsulation for transformations applied to _Shapes.
   Separates the "what it does" (action) from "how it does it" (traversal).'''   
   def __init__(self, transform):
      self.transform = transform


   def visit_Group(self, node, *args, **kwargs):
      for s in node:
         self.visit(s, *args, **kwargs)


   def visit_TransformLock(self, node, *args, **kwargs):
      if self.transform.func_name in node.lock_transforms:
         self._handle_transform_map(node, *args, **kwargs)
      else:
         for s in node:
            self.visit(s, *args, **kwargs)


   def visit__Shape(self, node, *args, **kwargs):
      node.points = self.transform(node.points, *args, **kwargs)


   ## private ##

   def _handle_transform_map(self, node, *args, **kwargs):
      tmp = self.transform
      t, p = node.get_transform(self.transform)(node.points, *args, **kwargs)
      self.transform = t
      for s in node:
         self.visit(s, *p)
      self.transform = tmp



#!/usr/bin/env python
from chiplotle import *

def envelope( ):
   plotter = instantiate_plotters( )[0]

   to_address = []
   from_address = []

   print ("\n\renter FROM ADDRESS (blank line to end):")

   finished = 0
   while finished == 0:
    
      line = raw_input("")
      if len(line) == 0:
         finished = 1
      else:
         from_address.append(line)

   print ("enter TO ADDRESS (blank line to end):")

   finished = 0
   while finished == 0:

      line = raw_input("")
      if len(line) == 0:
         finished = 1
      else:
         to_address.append(line)

   input = raw_input("\n\renter pen number for plotting FROM ADDRESS:\n")
   pen_num = int(input)

   plotter.write(SP(pen_num))

   raw_input("\n\rmove pen to top left of FROM: address field and hit return to start plotting...")
   print("plotting FROM: address...")

   for line in from_address:
      plotter.write(LB(line))
      plotter.write(LB("\n\r"))

   input = raw_input("\n\renter pen number for plotting TO ADDRESS:\n")
   pen_num = int(input)

   plotter.write(SP(pen_num))

   raw_input("\n\rmove pen to top left of TO ADDRESS field and hit return to start plotting...")
   print("plotting TO ADDRESS...")

   num_lines = len(to_address)
   line_num = 0

   for line in to_address:
      plotter.write(LB(line))
      plotter.write(LB("\n\r"))

   plotter.write(SP(0))
   print("done!")


if __name__ == '__main__':
   envelope( )



from chiplotle.geometry.core.path import Path
from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.core import errors
from py.test import raises

def test_path_add_01( ):
   '''A Path and an int cannot be added.'''
   assert raises(TypeError, 'Path([(1, 2), (3, 4)]) + 3')


def test_path_add_02( ):
   '''A Path and a float cannot be added.'''
   assert raises(TypeError, 'Path([(1, 2), (3, 4)]) + 3.2')


def test_path_radd_02( ):
   '''A float and a Path cannot be added.'''
   assert raises(TypeError, '3.2 + Path([(1, 2), (3, 4)])')


def test_path_add_03( ):
   '''A Path and a Coordinate can be added.'''
   a = Path([(1, 2), (3, 4)])
   t = a + Coordinate(1, 2)
   assert t is not a
   assert isinstance(t, Path)
   assert t == Path([(2, 4), (4, 6)])


def test_path_radd_03( ):
   '''A Coordinate and a Path can be added.'''
   a = Path([(1, 2), (3, 4)])
   t = Coordinate(1, 2) + a
   assert t is not a
   assert isinstance(t, Path)
   assert t == Path([(2, 4), (4, 6)])


def test_path_add_04( ):
   '''A Path and a duple cannot be added.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 'a + (1, 2)')


def test_path_radd_04( ):
   '''A duple and a Path cannot be added.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, '(1, 2) + a')


def test_path_add_05( ):
   '''A 2D Path and a triple cannot be added.'''
   a = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 'a + (1, 2, 3)')


def test_path_add_06( ):
   '''A Path and a Path cannot be added.'''
   a = Path([(1, 2), (3, 4)])
   b = Path([(2, 3)])
   assert raises(TypeError, 'a + b')


## in place addition __iadd__ ##

def test_path_iadd_01( ):
   '''A float and a Path cannot be added.'''
   t = Path([(1, 2), (3, 4)])
   assert raises(TypeError, 't += 3.2')





from chiplotle.geometry.core.polygon import Polygon

def rectangle(width, height):
   corners = []
   corners.append((-width / 2.0, -height / 2.0))
   corners.append((-width / 2.0, height / 2.0))
   corners.append((width / 2.0, height / 2.0))
   corners.append((width / 2.0, -height / 2.0))
   corners.append((-width / 2.0, -height / 2.0))
   return Polygon(corners)



## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.tools import io
   e = rectangle(100, 200)
   e.filled = True
   assert isinstance(e, Polygon)
   print e.format
   io.view(e)



#!/usr/bin/env python


def compile_plotters():
   from chiplotle import plotters

   content = 'Chiplotle Known Plotters\n'
   content += '========================\n\n'

   ## get classes in plotters...
   for cls in dir(plotters):
      if not cls.startswith('__'):
         content += '.. autoclass:: chiplotle.plotters.%s\n' % cls
         content += '\t:members:\n'
         content += '\t:undoc-members:\n'
         content += '\t:show-inheritance:\n'
         ## show inherited members for generic plotter...
         if cls == 'Plotter':
            content += '\t:inherited-members:\n'
         content += '\n'


   file = open('../chapters/api/plotters.rst', 'w')

   file.write(content)
   file.close()


## TODO: change this so that the tool modules are automatically discovered.
def compile_tools():
   from chiplotle import hpgltools
   from chiplotle import io
   from chiplotle import mathtools

   content = 'Chiplotle Tools\n'
   content += '================\n\n'

   content += 'HPGL Tools\n'
   content += '-------------\n\n'
   content += _autofunction(hpgltools)

   content += 'Input-output tools\n'
   content += '---------------------\n\n'
   content += _autofunction(io)

   content += 'Math tools\n'
   content += '--------------\n\n'
   content += _autofunction(mathtools)

   file = open('../chapters/api/tools.rst', 'w')
   file.write(content)
   file.close()

def compile_geometry():
   from chiplotle.geometry import core
   from chiplotle.geometry import shapes
   from chiplotle.geometry import transforms

   content = 'Chiplotle Geometry / Shapes\n'
   content += '============================\n\n'

   content += 'Shapes\n'
   content += '--------\n\n'
   content += _autofunction(shapes)

   content += 'Transforms\n'
   content += '-------------\n\n'
   content += _autofunction(transforms)

#   content += 'Core\n'
#   content += '-------------\n\n'
#   content += _autofunction(core)

   
   file = open('../chapters/api/geometry.rst', 'w')
   file.write(content)
   file.close()


## helpers ##

def _autofunction(module):
   modulename = module.__name__
   ret = []
   for cls in dir(module):
      if not cls.startswith('_'):
         af = '.. autofunction:: %s.%s' % (modulename, cls)
         ret.append(af)
   return '\n'.join(ret) + '\n\n'


if __name__ == '__main__':
   compile_plotters()
   compile_tools()
   compile_geometry()



from chiplotle import *
from chiplotle.hpgl.commands import *
from chiplotle.tools.hpgltools import transpose

def test_transpose_CI( ):
   '''Transpose correctly ignores CI attributes.'''
   t = CI(1, 20)
   transpose(t, (1.5, -2.5))
   assert t.radius == 1
   assert t.chordangle == 20 

def test_transpose_EA( ):
   '''Transpose works on single EA.'''
   t = EA((1, 2))
   transpose(t, (1.5, -2.5))
   assert t.xy == Coordinate(2.5, -0.5)

def test_transpose_ER( ):
   '''Transpose correctly ignores relative positions in ER.'''
   t = ER((1, 2))
   transpose(t, (1.5, -2.5))
   assert t.xy == Coordinate(1, 2)

def test_transpose_PU( ):
   '''Transpose ignores PU because behavior of PU depends on previous 
   PA or PR.'''
   t = PU([(1, 2)])
   transpose(t, (1.5, -2.5))
   assert t.xy == CoordinateArray([(1, 2)])

def test_transpose_PD( ):
   '''Transpose ignores PD.'''
   t = PD([(1, 2)])
   transpose(t, (1.5, -2.5))
   assert t.xy == CoordinateArray([(1, 2)])

def test_transpose_PA( ):
   '''Transpose works on a single PA.'''
   t = PA([(1, 2)])
   transpose(t, (1.5, -2.5))
   assert t.xy == CoordinateArray([(2.5, -0.5)])

def test_transpose_PR( ):
   '''Transpose correctly ignores relative positions in PR.'''
   t = PR([(1, 2)])
   transpose(t, (1.5, -2.5))
   assert t.xy == CoordinateArray([(1, 2)])

def test_transpose_RA( ):
   '''Transpose works on single RA.'''
   t = RA((1, 2))
   transpose(t, (1.5, -2.5))
   assert t.xy == Coordinate(2.5, -0.5)

def test_transpose_RR( ):
   '''Transpose correctly ignores relative positions in RR.'''
   t = RR((1, 2))
   transpose(t, (1.5, -2.5))
   assert t.xy == Coordinate(1, 2)



from chiplotle.hpgl.abstract.hpgl import _HPGL

class _HPGLPrimitive(_HPGL):

   _terminator = ';'


   @property
   def format(self):
      return '%s%s' % (self._name, _HPGLPrimitive._terminator)

   ### OVERRIDES ###

   ### TODO: [VA] make this simpler. remove all but the name?
   def __repr__(self):
      attributes = [ ]
      for a in dir(self):
         if not a.startswith('_'):
            if not callable(getattr(self, a)):
               #if a not in ('x', 'y', 'format', 'terminator'):
               if a not in ('x', 'y', 'format'):
                  attributes.append( '%s=%s' %(a, str(getattr(self, a))) )
      result = '%s(%s)' % (self._name, ', '.join(attributes))
      return result



"""
 *  This file is part of chiplotle.
 *
 *  http://music.columbia.edu/cmc/chiplotle
"""

from chiplotle.plotters.drawingplotter import _DrawingPlotter

class HP7475A(_DrawingPlotter):
   def __init__(self, ser, **kwargs):
      self.allowedHPGLCommands = tuple(['\x1b.', 'AA','AR','CA','CI','CP',
         'CS','DC','DF','DI','DP','DR','DT','EA','ER','EW','FT','IM','IN',
	      'IP','IW','LB','LT','OA','OC','OD','OE','OF','OH','OI','OO','OP',
         'OS','OW','PA','PD','PR','PS','PT','PU','RA','RO','RR','SA','SC',
         'SI','SL','SM','SP','SR','SS','TL','UC','VS','WG','XT','YT'])
      _DrawingPlotter.__init__(self, ser, **kwargs)
      self.type = "HP7475A"





from chiplotle.tools.mathtools.pascal_row import pascal_row
import numpy

def bezier_interpolation(control_points, points_to_compute, weight):
      '''Computes Bezier interpolations from given `control_points`.
      This uses the generalized formula for bezier curves:
      http://en.wikipedia.org/wiki/B%C3%A9zier_curve#Generalization

      - `control_points` : A list of (x, y) control points.
      - `points_to_compute`: An int of the number of points to compute.
      - `weight`: A list of weights for control points.
      '''

      control_points = numpy.array(control_points)
      number_points = len(control_points)
      combinations = pascal_row(number_points-1)
      ts = [t / float(points_to_compute) for t in range(points_to_compute + 1)]

      result = [ ]
      for t in ts:
         tpowers = (t**i for i in range(number_points))
         upowers = reversed([(1-t)**i for i in range(number_points)])
         coefs = numpy.array([c*a*b for c,a,b in zip(combinations,tpowers,upowers)])
         num = numpy.column_stack((coefs*weight*numpy.column_stack(control_points))).sum(axis=0)
         den = (coefs*weight).sum(axis=0)
         if den == 0:
            den = 1
         point = tuple(num/den)
         result.append(point)
      return result



from chiplotle.core.cfg.read_config_file import read_config_file

def get_config_value(var_name):
   try:
      result = read_config_file( )[var_name]
      return result
   except KeyError:
      msg = ""
      msg +="\n* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\a\n"
      msg +="The configuration variable '%s'\n " % var_name
      msg +="could not be found in the Chiplotle configuration file. \n"
      msg +="This could be due to an update in Chiplotle that modified\n"
      msg +="the configuration parameters of the file.\n" 
      msg +="To fix,  please delete your configuration file and run\n"
      msg +="Chiplotle again.\n"
      msg +="* * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\a\n"
      raise ImportError(msg)
      



from chiplotle import *
from py.test import raises

def test_coordinate__add__01( ):
   '''Two Coordinates can be added.'''
   a = Coordinate(1, 2)
   b = Coordinate(3, 4)
   t = a + b
   assert isinstance(t, Coordinate)
   assert t is not a
   assert t is not b
   assert t == Coordinate(4, 6)


def test_coordinate__add__02( ):
   '''A Coordinate and an int scalar cannot be added.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 't = a + 4')


def test_coordinate__radd__02( ):
   '''An int scalar and a  Coordinate cannot be added.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 't = 4 + a')


def test_coordinate__add__03( ):
   '''A Coordinate and a float scalar cannot be added.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 't = a + 4.2')


def test_coordinate__radd__03( ):
   '''A float scalar and a Coordinate can be added.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 't = 4.2 + a')


def test_coordinate__add__04( ):
   '''A Coordinate and a tuple pair cannot be added.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, 'a + (3, 4)')


def test_coordinate__radd__04( ):
   '''A tuple pair and a Coordinate cannot be added.'''
   a = Coordinate(1, 2)
   assert raises(TypeError, '(3, 4) + a')


def test_coordinate__add__05( ):
   '''A Coordinate and a CoordinateArray can be added.'''
   a = Coordinate(1, 2)
   b = CoordinateArray([(3, 4), (5, 6)])
   t = a + b
   assert isinstance(t, CoordinateArray)
   assert t[0] == Coordinate(4, 6)
   assert t[1] == Coordinate(6, 8)


def test_coordinate__radd__05( ):
   '''A CoordinateArray and a Coordinate can be added.'''
   a = Coordinate(1, 2)
   b = CoordinateArray([(3, 4), (5, 6)])
   t = b + a
   assert isinstance(t, CoordinateArray)
   assert t[0] == Coordinate(4, 6)
   assert t[1] == Coordinate(6, 8)


def test_coordinate__add__06( ):
   '''A 2D Coordinate and a triple cannot be added.'''
   a = Coordinate(1, 2)
   b = (3, 4, 5)
   assert raises(TypeError, 'a + b')

def test_coordinate__radd__06( ):
   '''A triple and a 2D Coordinate cannot be added.'''
   a = Coordinate(1, 2)
   b = (3, 4, 5)
   assert raises(TypeError, 'b + a')





import serial

def scan_serial_ports_from_list(portlist):
   
   result = { }
   ## keep track of previously opened ports to avoid including aliases
   ## in our list...
   ports_opened = [ ]
   for i, port in enumerate(portlist):
      try:
         s = serial.Serial(port)
         ports_opened.append(s)
         result[i] = s.portstr
         #result[i] = s
         #s.close( )   # explicit close 'cause of delayed GC in java
      except serial.SerialException:
         pass
   ## gracefully close all open ports...
   for port in ports_opened:
      port.close( )
   return result



from chiplotle.hpgl.commands import PU, PA, PD
from chiplotle.geometry.core.coordinatearray import CoordinateArray

def convert_coordinates_to_hpgl_absolute_path(coords):
   '''Converts an iterator of lists of coordinates
   e.g., [<x1, y1>, <x2, y2>, <x3, y3>, ...]
   into a list of PA, PD and PU HPGL commands. 
   '''
   if not isinstance(coords, (list, tuple, CoordinateArray)):
      raise TypeError('`coords` must be a list of coordinates or CoordinateArray.')

   coords = [list(c) for c in coords]
   result = [ ]
   result.append(PU( ))
   result.append(PA([coords[0]]))
   result.append(PD( ))
   ## this denies the possibility of paths with one coord.
   #result.append(PA(coords[1:])) 
   result.append(PA(coords))
   result.append(PU( ))
   return result



from chiplotle.hpgl.commands import FT
from py.test import raises

def test_FT_01( ):
   t = FT( )
   assert t.type is None
   assert t.space is None
   assert t.angle is None
   assert t.format == 'FT;'
   

def test_FT_02( ):
   t = FT(2)
   assert t.type == 2
   assert t.format == 'FT2;'


def test_FT_03( ):
   t = FT(2, .23)
   assert t.type == 2
   assert t.space == 0.23
   assert t.format == 'FT2,0.23;'


def test_FT_04( ):
   t = FT(2, .23, 80.5)
   assert t.type == 2
   assert t.space == 0.23
   assert t.angle == 80.5
   assert t.format == 'FT2,0.23,80.5;'


def test_FT_05( ):
   '''Mandatory parameters missing raises Warning at format time.'''
   t = FT( )
   t.space = .23
   assert t.type is None
   assert t.space == 0.23
   assert t.angle is None
   assert raises(Warning, 't.format')




from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.core.path import Path
from chiplotle.tools.mathtools.interpolate_linear import interpolate_linear

def path_linear(coords, interpolation_unit):
   '''Returns a path with linearly interpolated segments. 
   Visually the result is the same as a plain path, but this is
   useful as an intermediate step in constructing more interesting
   shapes by deforming this one.
   
   - `coords` is a CoordinateArray.
   - `interpolation_unit` is the magnitude of the path segments created.
      Think of it as the sampling period in digital recording.
      If `interpolation_unit` > coord[i] - coord[i-1], the 
      coord[i] - coord[i-1] segment is not further segmented.
   '''
   coords = CoordinateArray(coords)
   
   def units_per_path_segment(coords, interpolation_unit):
      result = [ ]
      diffs = coords.difference
      for i in range(len(diffs)):
         segs = diffs[i].magnitude / float(interpolation_unit)
         coord_pair = (coords[i], coords[i + 1])
         if segs < 1:
            result.append((coord_pair, 1))
         else:
            result.append((coord_pair, int(round(segs))))
      return result


   divs = units_per_path_segment(coords, interpolation_unit)
   result = [ ]
   for cp, n in divs:
      newcoords = [interpolate_linear(cp[0], cp[1], float(i) / n) for i in range(n)]
      result.extend(newcoords)

   return Path(result)



## demo
if __name__ == '__main__':
   from chiplotle import *
   import random

   coords = CoordinateArray([random.randint(0, 1000) for i in range(10)])
   #coords = CoordinateArray([0, 0, 300, 0, 300, 300, 0, 300, 0, 0])
   p = path_linear(coords, 100)

   circs =  [ ]
   for coord in p.points:
      c = circle(10, 8)
      offset(c, coord)
      circs.append(c)


   io.view(group([p] + circs))



from chiplotle.hpgl.commands import SetHandshakeMode
from py.test import raises

def test_dci_sethandshakemode_01( ):
   '''SetHandshakeMode can take no parameters.'''
   t = SetHandshakeMode( )

   assert t.format == '\x1b.P'


def test_dci_sethandshakemode_02( ):
   '''SetHandshakeMode can take None, 0, 1, 2 or 3 to set the mode.'''
   t = SetHandshakeMode(1)

   assert t.format == '\x1b.P1'


def test_dci_sethandshakemode_03( ):
   '''SetHandshakeMode cannot take a value other than None, 0, 1, 2 or 3.'''
   assert raises(ValueError, 't = SetHandshakeMode(32)')





from chiplotle import plotters

def instantiate_plotter_from_id(serial, id):
   '''Tries to instantiate a plotter that matches the given string `id`.
   
   - `serial` is a Serial instance.
   - `id` is a string of the plotter ID.
   '''
   ## massage id...
   id = id.replace('-', '').strip('\r')
   ## find a plotter within existing plotters that matches ID 
   ## and instantiate.
   for plt_str in dir(plotters):
      if id in plt_str:
         plotter = getattr(plotters, plt_str)(serial)
         return plotter



from chiplotle import *
from chiplotle.geometry.core.layersvisitor import LayersVisitor
from chiplotle.geometry.core.layer import Layer

def test_layervisitor_01():
   '''Shapes without layer are set to layer None, the default.'''
   r = rectangle(1000, 1000)
   v = LayersVisitor()
   v.visit(r)
   assert v.layers.keys() == [None]
   assert v.layers[None] == [r]

def test_layersvisitor_02():
   '''One paths with layer gorup.'''
   r1 = rectangle(1000, 1000)
   l = Layer([r1], 1)
   v = LayersVisitor()
   v.visit(l)
   assert v.layers.keys() == [1]
   assert v.layers[1] == [r1]

def test_layersvisitor_03():
   '''Nested Layers work as expected.'''
   r1 = rectangle(1000, 1000)
   r2 = rectangle(500, 500)
   t3 = isosceles(500, 200)

   l1 = Layer([r1], 1)
   l2 = Layer([l1, r2], 2)
   l3  = Layer([t3, l2], 3)

   v = LayersVisitor()
   v.visit(l3)

   assert v.layers.keys() == [1, 2, 3]
   assert v.layers[1] == [r1]
   assert v.layers[2] == [r2]
   assert v.layers[3] == [t3]

#   r1   r2   t3
#   .     .   .
#   l1   .   .
#    .  .   .
#     l2   .
#      .  .
#       l3

def test_layersvisitor_04():
   '''Nested Layers and Groups work as expected.'''
   r1 = rectangle(1000, 1000)
   r2 = rectangle(500, 500)
   t3 = isosceles(500, 200)

   l1 = Layer([r1], 1)
   g1 = group([l1, r2])
   l3 = Layer([t3, g1], 3)

   v = LayersVisitor()
   v.visit(l3)

   assert v.layers.keys() == [1, 3]
   assert v.layers[1] == [r1]
   assert v.layers[3] == [t3, r2]

#   r1   t3   r3
#   .     .   .
#   l1   .   .
#    .  .   .
#     g1   .
#      .  .
#       l3

def test_layervisitor_05():
   '''Multiple nestings work as expected.'''
   c1, c2, c3 = [circle(1000 * i) for i in range(1, 4)]
   r = rectangle(1000, 1000)
   t = isosceles(1000, 2000)

   l_leaf = Layer([c1], 'leaf')
   g1 = Group([l_leaf, c2, c3])
   l_bottom = Layer([g1], 'bottom')

   l_med = Layer([l_bottom, r], 'med')
   
   l_top = Layer([l_med, t], 'top')
   
   v = LayersVisitor()
   v.visit(l_top)

   assert sorted(v.layers.keys()) == ['bottom', 'leaf', 'med', 'top']
   assert v.layers['top']     == [t]
   assert v.layers['med']     == [r]
   assert v.layers['bottom']  == [c2, c3]
   assert v.layers['leaf']    == [c1]

#   c1   c2 c3  r  t
#   .     . .   .  .
#   l_l  . .    .  .
#     . . .    .  .
#       g1    .  .
#       .    .  .
#      l_b  .  .
#        . .  .
#        l_m .
#         . .
#         l_t

## DEPRECATED ##
############################################################

#from chiplotle.geometry.core.layersvisitor import LayersVisitor
#
#def test_layersvisitor_01():
#   '''Simple paths with set layer.'''
#   r1 = rectangle(1000, 1000)
#   r1.layer = 1
#   v = LayersVisitor()
#   v.visit(r1)
#   assert v.layers.keys() == [1]
#   assert v.layers[1] == [r1]
#
#
#def test_layervisitor_01b():
#   '''Shapes without layer are set to layer None, the default.'''
#   r = rectangle(1000, 1000)
#   v = LayersVisitor()
#   v.visit(r)
#   assert v.layers[None] == [r]
#   assert v.layers.keys() == [None]
#
#
#def test_layersvisitor_02():
#   '''A group defines the layer of it's undelying Shapes, 
#   if the layer of these underlying shapes is set to None.
#   Shapes with their layer set override the Group layer.'''
#   r1 = rectangle(1000, 1000)
#   r1.layer = 1
#   r2 = rectangle(500, 500)
#   r2.layer = None
#
#   g = Group([r1, r2])
#   g.layer = 2
#
#   v = LayersVisitor()
#   v.visit(g)
#
#   assert v.layers.keys() == [1, 2]
#   assert v.layers[1] == [r1]
#   assert v.layers[2] == [r2]
#
#
#   
#def test_layersvisitor_03():
#   '''Nested Groups work as expected.'''
#   r1 = rectangle(1000, 1000)
#   r1.layer = 1
#   r2 = rectangle(500, 500)
#   r2.layer = None
#   t  = isosceles(500, 200)
#   t.layer = None
#
#   rg = Group([r1, r2])
#   rg.layer = 2
#   g = Group([t, rg])
#   g.layer = 3
#
#   v = LayersVisitor()
#   v.visit(g)
#
#   assert v.layers[1] == [r1]
#   assert v.layers[2] == [r2]
#   assert v.layers[3] == [t]
#
#
#def test_layervisitor_04():
#   '''Multiple nestings work as expected.'''
#   circs = [circle(1000 * i) for i in range(1, 3)]
#   circs[0].layer = 'leaf'
#   gcircs = Group(circs)
#   gcircs.layer = 'bottom'
#
#   r = rectangle(1000, 1000)
#   med = Group([gcircs, r])
#   med.layer = 'med'
#   
#   t = isosceles(1000, 2000)
#   top = Group([med, t])
#   top.layer = 'top'
#   
#   v = LayersVisitor()
#   v.visit(top)
#
#   assert set(v.layers.keys()) == set(['bottom', 'med', 'top', 'leaf'])
#   assert v.layers['top']     == [t]
#   assert v.layers['med']     == [r]
#   assert v.layers['bottom']  == circs[1:]
#   assert v.layers['leaf']    == circs[0:1]



from chiplotle import *
from chiplotle.hpgl.commands import *

def test_pens_updown_to_papr_01( ):
   g = [PA([(1, 2)]), PU([(1, 1), (2, 2)])]
   t = hpgltools.pens_updown_to_papr(g)
   assert len(t) == 3
   assert t[0] == PA([(1, 2)])
   assert t[1] == PU( )
   assert t[2] == PA([(1, 1), (2, 2)])


def test_pens_updown_to_papr_02( ):
   g = [PU([(1, 1), (2, 2)])]
   t = hpgltools.pens_updown_to_papr(g)
   assert len(t) == 2
   assert t[0] == PU( )
   assert t[1] == PA([(1, 1), (2, 2)])


def test_pens_updown_to_papr_03( ):
   '''Other HPGL commands are passed untouched.'''
   g = [PA([(0, 0)]), CI(100), PD([(1, 1), (2, 2)]), ER((10, 10))]
   t = hpgltools.pens_updown_to_papr(g)
   assert len(t) == 5
   assert t[0] == PA([(0, 0)])
   assert t[1] == CI(100)
   assert t[2] == PD( )
   assert t[3] == PA([(1, 1), (2, 2)])
   assert t[4] == ER((10, 10))


def test_pens_updown_to_papr_04( ):
   '''The function returns fresh new instances of PA and or PR.'''
   g = [PA(), PD([(1, 1)]), PD([(2, 2)]), PD([(3, 3)])]
   t = hpgltools.pens_updown_to_papr(g)
   assert len(t) == 7
   assert t[2] is not t[4]
   assert t[4] is not t[6]
   



from chiplotle import *
from chiplotle.hpgl.commands import *
from chiplotle.tools.hpgltools import scale

def test_scale_CI( ):
   '''Scale works on single CI.'''
   t = CI(1, 20)
   scale(t, 1.5)
   assert t.radius == 1.5
   assert t.chordangle == 20 ### chordangle is unmodified
   
def test_scale_ER( ):
   '''Scale works on single ER.'''
   t = ER((1, 2))
   scale(t, 1.5)
   assert t.xy == Coordinate(1.5, 3.0)

def test_scale_EA( ):
   '''Scale works on single EA.'''
   t = EA((1, 2))
   scale(t, 1.5)
   assert t.xy == Coordinate(1.5, 3.0)

def test_scale_PU( ):
   '''Scale works on single PU.'''
   t = PU([(1, 2)])
   scale(t, 1.5)
   assert t.xy == CoordinateArray([(1.5, 3.0)])

def test_scale_PD( ):
   '''Scale works on single PD.'''
   t = PD([(1, 2)])
   scale(t, 1.5)
   assert t.xy == CoordinateArray([(1.5, 3.0)])

def test_scale_PA( ):
   '''Scale works on single PA.'''
   t = PA([(1, 2)])
   scale(t, 1.5)
   assert t.xy == CoordinateArray([(1.5, 3.0)])

def test_scale_PR( ):
   '''Scale works on single PR.'''
   t = PR([(1, 2)])
   scale(t, 1.5)
   assert t.xy == CoordinateArray([(1.5, 3.0)])

def test_scale_RA( ):
   '''Scale works on single RA.'''
   t = RA((1, 2))
   scale(t, 1.5)
   assert t.xy == Coordinate(1.5, 3.0)
   
def test_scale_RR( ):
   '''Scale works on single RR.'''
   t = RR((1, 2))
   scale(t, 1.5)
   assert t.xy == Coordinate(1.5, 3.0)
   

def test_scale_list_01( ):
   t = [PA([(1, 2)]), PU([(3, 4)])]
   scale(t, 1.5)
   assert t[0].xy == CoordinateArray([(1.5, 3)])
   assert t[1].xy == CoordinateArray([(4.5, 6)])




from chiplotle.geometry.core.group import Group
from chiplotle.geometry.core.path import Path
from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.tools.mathtools.difference import difference
from chiplotle.geometry.transforms.transformvisitor import TransformVisitor
import random

def perpendicular_noise(shape, value):
   '''Distort shape by adding noise perpendiculary to the path.
   This is an in-place destructive transformation; no new shapes are created.

   - `shape` is the shape to be noisified.
   - `value` must be a scalar defining the range of the noise 
      for displacement.
   '''
   def perpnoise(coords, value):
      result = [ ]
      d_coords = difference(coords)
      for coord, d_coord in zip(coords[:-1], d_coords):
         wiggle = random.randrange(-value, value)
         perp = ~d_coord 
         xy = coord + perp / perp.magnitude * wiggle         
         result.append(xy)
      return CoordinateArray(result)

   t = TransformVisitor(perpnoise)
   t.visit(shape, value)

      
      
   
## RUN DEMO CODE
if __name__ == "__main__":
   from chiplotle.geometry.shapes.circle import circle
   from chiplotle.geometry.core.coordinate import Coordinate
   from chiplotle.tools import io
   c1 = circle(1000, 200)
   c2 = circle(800, 200)
   c2 += Coordinate(2000, 2000)
   perpendicular_noise(c1, 360)
   perpendicular_noise(c2, 60)
   g = Group([c1, c2])
   io.view(g)




from chiplotle.geometry.shapes.circle import circle
from chiplotle.geometry.transforms.scale import scale


def ellipse(width, height, segments = 36):  
   '''
   Constructs an ellipse with the given width, height, and number of segments.
   '''
   r = circle(1, segments)
   scale(r, (width / 2.0, height / 2.0))
   return r

#   two_pi = math.pi * 2.0
#   
#   rads_incr = two_pi / float(segments)
#   half_width = width * 0.5
#   half_height = height * 0.5
#   
#   rads = 0.0
#   
#   ellipse_points = []
#   
#   while rads < two_pi: 
#      sin = math.sin(rads);
#      cos = math.cos(rads);
#
#      point_x = (half_width * cos);
#      point_y = (half_height * sin);
#
#      ellipse_points.append((point_x, point_y))
#      
#      rads += rads_incr
# 
#   result = Polygon(ellipse_points)
#   return result


## RUN DEMO CODE

if __name__ == '__main__':

   from chiplotle import *
   from chiplotle.geometry import *

   import math

   two_pi = math.pi * 2.0

   g = []

   for a in range(0,7):
      e = ellipse(5000, 1000, 500)
      assert isinstance(e, Polygon)
      rotate(e, (two_pi / 7) * a)
      g.append(e)
   
   io.view(Group(g))




from chiplotle.geometry.core.polygon import Polygon
from chiplotle.geometry.core.group import Group
from chiplotle.tools.mathtools.lcm import lcm
import math
from fractions import gcd
      
def star_outline(width, height, num_points = 5):  
   '''Constructs a star shape in outline.  '''
   corners = []
   pi_div_180 = math.pi / 180.0
   half_width = width * 0.5
   half_height = height * 0.5

   degrees = degrees_offset = 90
   even = True
   
   degrees_incr = 360.0 / float(num_points * 2)
   quarter_width = half_width * 0.5
   quarter_height = half_height * 0.5
   
   while degrees < 360.0 + degrees_offset: 
      alpha = degrees * pi_div_180
      sin_alpha = math.sin(alpha);
      cos_alpha = math.cos(alpha);

      if even:
         w_multi = half_width
         h_multi = half_height
         even = False
      else:
         w_multi = quarter_width
         h_multi = quarter_height
         even = True
         
      point_x = (w_multi * cos_alpha);
      point_y = (h_multi * sin_alpha);

      corners.append((point_x, point_y))
      
      degrees += degrees_incr

   corners.append(corners[0])
   return Polygon(corners)
   


## RUN DEMO CODE
if __name__ == '__main__':
   from chiplotle.tools import io
   from chiplotle.geometry.shapes.star_outline import star_outline
   from chiplotle.geometry.core.group import Group
   
   gr1 = Group()
   
   for i in range(5, 10):
      st = star_outline(100 * (i * i * i), 100 * (i * i * i), num_points = i)
      gr1.append(st)
   
   io.view(gr1)



from chiplotle import *
from py.test import raises

def test_coordinate__mul__01( ):
   '''A Coordinate can be multiplied with a scalar.'''
   a = Coordinate(1, 2)
   b = 2.5
   t = a * b
   assert isinstance(t, Coordinate)
   assert t is not a
   assert t == Coordinate(2.5, 5)


def test_coordinate__mul__02( ):
   '''A Coordinate cannot be multiplied with a pair.'''
   a = Coordinate(2, 3)
   assert raises(TypeError, 'a * (2, 3)')


def test_coordinate__mul__03( ):
   '''A Coordinate can be multiplied with another Coordinate.'''
   a = Coordinate(2, 3)
   t = a * Coordinate(2, 3)
   assert isinstance(t, Coordinate)
   assert t == Coordinate(4, 9)






"""
 *  This file is part of chiplotle.
 *
 *  http://music.columbia.edu/cmc/chiplotle
"""
from chiplotle.plotters.drawingplotter import _DrawingPlotter

class DPX3300(_DrawingPlotter):
   def __init__(self, ser, **kwargs):
      self.allowedHPGLCommands = tuple(['\x1b.', 'AA','AR','BL','CA','CC',
         'CI','CP','CS','CT','DC','DF','DI','DL','DP','DR','DT','EA','EP',
         'ER','ES','EW','FP','FS','FT','IM','IN','IP','IW','LB','LO','LT',
         'OA','OC','OD','OE','OF','OH','OI','OL','OO','OP','OS','OT','OW',
         'PA','PB','PD','PM','PU','PR','PT','RA','RO','RR','SA','SC','SI',
         'SL','SM','SP','SR','SS','TL','UC','UF','VS','WG','XT','YT'])
      _DrawingPlotter.__init__(self, ser, **kwargs)
      self.type = "DPX-3300"




from chiplotle import *

def test_parse_hpgl_string_01( ):
   '''Coma separated integer arguments numbers are handled properly.'''
   s = 'IN;PA0,0PD1,2;PU3,4PD'
   t = hpgltools.parse_hpgl_string(s)
   assert t == ['IN', 'PA0,0', 'PD1,2', 'PU3,4', 'PD']

def test_parse_hpgl_string_02( ):
   '''Coma separated floating point numbers are handled correctly.'''
   s = 'IN;PA0,0.1PD1.33,2.0;PU3,4PD'
   t = hpgltools.parse_hpgl_string(s)
   assert t == ['IN', 'PA0,0.1', 'PD1.33,2.0', 'PU3,4', 'PD']

def test_parse_hpgl_string_03( ):
   '''Negative values are handled correnctly.''' 
   s = 'IN;SP2;PU;PA169.33,-0.00;PD;PA169.33,-1.00,170.22,6.32,170.86,12.67;PU;'
   t = hpgltools.parse_hpgl_string(s)
   assert t == ['IN', 'SP2', 'PU', 'PA169.33,-0.00', 'PD', 
      'PA169.33,-0.00,170.22,6.32,170.86,12.67', 'PU']

def test_parse_hpgl_string_04( ):
   '''Commands stringed together (with no ; separator) are handled correctly.'''
   s = 'INSP2PUPA2,-1PDRR100PU'
   t = hpgltools.parse_hpgl_string(s)
   assert t == ['IN', 'SP2', 'PU', 'PA2,-1', 'PD', 'RR100', 'PU']


## DCI (escape) commands ##

def test_parse_hpgl_string_03( ):
   '''Plotter On DCI is captured.'''
   s = '\x1b.YIN;PA0,0;'
   t = hpgltools.parse_hpgl_string(s)
   assert t == ['\x1b.Y', 'IN', 'PA0,0']


def test_parse_hpgl_string_03b( ):
   '''Plotter On DCI is captured.'''
   s = '\x1b.(IN;PA0,0;'
   t = hpgltools.parse_hpgl_string(s)
   assert t == ['\x1b.(', 'IN', 'PA0,0']




#import serial

def scan_serial_ports_windows( ):
   from chiplotle.tools.serialtools import scan_serial_ports_from_list
   return scan_serial_ports_from_list(range(256))
#   result = { }
#   ## keep track of previously opened ports to avoid including aliases
#   ## in our list...
#   ports_opened = [ ]
#   for i in range(256):
#      try:
#         s = serial.Serial(i)
#         ports_opened.append(s)
#         result[i] = s.portstr
#         #result[i] = s
#         #s.close( )   # explicit close 'cause of delayed GC in java
#      except serial.SerialException:
#         pass
#   ## gracefully close all open ports...
#   for port in ports_opened:
#      port.close( )
#   return result
   



#!/usr/bin/env python

from chiplotle import *
from chiplotle.tools.plottertools import instantiate_virtual_plotter

'''
Demonstrates the use of a virtual plotter with a specific plotter definition.

You must have hp2xx installed for io.view() to work!
'''

'''
    compute size of paper:
    there are 40 plotter units per mm
    HP7550A reference lists 11x17 (ANSI B) max plot dimensions as:
    254 x 411mm
'''

paper_width = 411 * 40
paper_length = 254 * 40

plotter = instantiate_virtual_plotter(type="HP7550A", 
    left_bottom = Coordinate(0,0), 
    right_top = Coordinate(paper_width, paper_length) )

plotter.margins.soft.draw_outline()
plotter.goto_center()
plotter.select_pen(1)
plotter.write(hpgl.CI(1000))
plotter.select_pen(2)
plotter.write(hpgl.CI(500))
plotter.select_pen(3)
plotter.write(hpgl.CI(250))
plotter.select_pen(4)
plotter.write(hpgl.CI(125))

io.view(plotter)




from chiplotle import *
from chiplotle.hpgl.commands import *
from py.test import raises

def test_convert_coordinates_to_absolute_hpgl_path_01( ):
   '''Coordinates can be generic tuples.'''
   c = [(1, 2), (3, 4), (5, 6)]
   t = hpgltools.convert_coordinates_to_hpgl_absolute_path(c)
   assert t[0] == PU( )
   assert t[1] == PA([c[0]])
   assert t[2] == PD( )
   assert t[3] == PA(c[:])


def test_convert_coordinates_to_absolute_hpgl_path_02( ):
   '''Coordinates can be Coordinates.'''
   c = [Coordinate(1, 2), Coordinate(3, 4), Coordinate(5, 6)]
   t = hpgltools.convert_coordinates_to_hpgl_absolute_path(c)
   assert t[0] == PU( )
   assert t[1] == PA([c[0].xy])
   assert t[2] == PD( )
   assert t[3] == PA(c)


def test_convert_coordinates_to_absolute_hpgl_path_03( ):
   '''The argument can be a CoordinateArray'''
   c = CoordinateArray([Coordinate(1, 2), Coordinate(3, 4), Coordinate(5, 6)])
   t = hpgltools.convert_coordinates_to_hpgl_absolute_path(c)
   assert t[0] == PU( )
   assert t[1] == PA([list(c[0])])
   assert t[2] == PD( )
   assert t[3] == PA([list(x) for x in c])


def test_convert_coordinates_to_absolute_hpgl_path_04( ):
   '''Argument must be a list of coordinate pairs..'''
   c = [1, 2, 3, 4]
   assert raises(TypeError, 'hpgltools.convert_coordinates_to_hpgl_absolute_path(c)')





from chiplotle.hpgl.commands import *
import py.test

def test_AA_01( ):
   t = AA((0,0), 180)

   assert t.angle == 180
   assert t.chordtolerance is None
   assert t.format == 'AA0,0,180.00;'


def test_AA_02( ):
   t = AA((0,0), 180, 45)

   assert t.angle == 180
   assert t.chordtolerance == 45
   assert t.format == 'AA0,0,180.00,45.00;'


def test_AA_03( ):
   '''AA must take at least two arguments: position and angle.'''
   assert py.test.raises(TypeError, 'AA((0, 0))')


def test_AA_format_01( ):
   '''Ints format as ints.'''
   t = AA((0,0), 180)

   assert t.format == 'AA0,0,180.00;'


def test_AA_format_02( ):
   '''Floats format as floats.'''
   t = AA((0,0.0), 180)

   assert t.format == 'AA0.00,0.00,180.00;'
   

def test_AA_angle_01( ):
   '''Angle must be between -360 and 360.'''
   assert py.test.raises(ValueError, 'AA((0, 0), 1200)')

## eq ##

def test_AA__eq__01( ):
   assert AA((1, 2), 120) == AA((1, 2), 120)



from chiplotle.geometry.shapes.path_linear import path_linear
from chiplotle.geometry.transforms.perpendicular_displace \
   import perpendicular_displace

def line_displaced(start_coord, end_coord, displacements):
   '''Returns a Path defined as a line spanning points `start_coord` and
   `end_coord`, displaced by scalars `displacements`. 
   The number of points in the path is determined by the lenght of 
   `displacements`.
   '''
   unit = (end_coord - start_coord).magnitude / len(displacements)

   path = path_linear((start_coord, end_coord), unit)
   perpendicular_displace(path, displacements)
   
   return path

   
   

if __name__ == '__main__':
   from chiplotle import *
   import math

   disp = [math.sin(i**0.7 / 3.14159 * 2) * 100  for i in range(200)]
   line = line_displaced(Coordinate(0, 0), Coordinate(1000, 1000), disp)

   io.view(line)





class Visitor(object):
   
   def visit(self, node, *args, **kwargs):
      for cls in node.__class__.__mro__:
         meth_name = 'visit_' + cls.__name__
         meth = getattr(self, meth_name, None)
         if meth:
            break
      else:
         meth = self.visit_generic
      return meth(node, *args, **kwargs)


   def visit_generic(self, node, *args, **kwargs):
      print '*** In generic Visitor function.'
      print 'Node: %s' % node




from chiplotle.geometry.core.coordinate import Coordinate
from chiplotle.tools.hpgltools.get_all_coordinates import get_all_coordinates

def get_bounding_box(arg):
   '''Returns the pair of coordinate pairs outlining the bounding box of
   the given HPGL drawing.'''
   if not isinstance(arg, (list, tuple)):
      raise TypeError('arg must be list or tuple')

   min_x = min_y = 1000000.0
   max_x = max_y = -1000000.0
   
   coords = get_all_coordinates(arg)

   if len(coords) == 0:
      return None

   for c in coords:
      ## x...
      if c.x > max_x:
         max_x = c.x
      if c.x < min_x:
         min_x = c.x
      ## y...
      if c.y > max_y:
         max_y = c.y
      if c.y < min_y:
         min_y = c.y

   return (Coordinate(min_x, min_y), Coordinate(max_x, max_y))



"""
 *  This file is part of chiplotle.
 *
 *  http://music.columbia.edu/cmc/chiplotle
"""
from chiplotle.plotters.drawingplotter import _DrawingPlotter

class HP7550A(_DrawingPlotter):
   def __init__(self, ser, **kwargs):
      self.allowedHPGLCommands = tuple(['\x1b.', 'AA','AF','AH','AP','AR',
         'BF','BL','CA','CC','CI','CM','CP','CS','CT','CV','DC','DF','DI',
         'DL','DP','DR','DS','DT','EA','EP','ER','ES','EW','FP','FS','FT',
         'GC','IM','IN','IP','IV','IW','KY','LB','LO','LT','NR','OA','OC',
         'OD','OE','OF','OG','OH','OI','OK','OL','OO','OP','OS','OT','OW',
         'PA','PB','PD','PG','PM','PR','PT','PU','RA','RO','RP','RR','SA',
         'SC','SI','SL','SM','SP','SR','SS','TL','UC','UF','VS','WD','WG',
         'XT','YT'])
      _DrawingPlotter.__init__(self, ser, **kwargs)
      self.type = "HP7550A"




from chiplotle.geometry.core.coordinatearray import CoordinateArray
from chiplotle.geometry.core.shape import _Shape
import copy

class Group(_Shape):
   '''A group collects together multiple _Shapes, so they
   can be treated as a single object. 
   The elements in a group are stored in order, like a list, so that
   order-specific operation can be performed on groups. 
   i.e., Groups are lists, not sets.
   '''
   def __init__(self, shapes=None):
      _Shape.__init__(self)
      self._shapes = [ ]
      if shapes is None: shapes = [ ]
      self.extend(shapes)


   ## PUBLIC PROPERTIES ##

   @property
   def points(self):
      '''Returns a flat list of all the Coordinates that form this shape.
      This property is useful in computing some property of the shape based
      on all it's points. e.g., centroid, bounding box, etc. '''
      coords = [ ]
      for shape in self:
         coords += list(shape.points)
      return CoordinateArray(coords)
      #return coords


   ## PUBLIC METHODS ##

   def append(self, arg):
      self._check_init_shape(arg)
      self._shapes.append(arg)

   def extend(self, arg):
      self._check_init_shapes(arg)
      self._shapes.extend(arg)

   def insert(self, i, arg):
      self._check_init_shape(arg)
      self._shapes.insert(i, arg)

   def remove(self, arg):
      self._shapes.remove(arg)

   def pop(self, indx=-1):
      result = self._shapes.pop(indx)
      return result


   ## private properties ##

#   @property
#   def _infix_commands(self):
#      result = [ ]
#      for shape in self:
#         result += shape._subcommands
#      return result


   ## private methods ##

   def _check_init_shape(self, shape):
      if not isinstance(shape, _Shape):
         raise TypeError('shape must be an _Shape object.')

   def _check_init_shapes(self, shapes):
      for s in shapes:
         self._check_init_shape(s)


   ## OVERRIDES ##

   def __delitem__(self, indx):
      del(self._shapes[indx])

   def __getitem__(self, indx):
      return self._shapes[indx]

   def __len__(self):
      return len(self._shapes)

   def __setitem__(self, i, arg):
      if isinstance(i, int):
         self._check_init_shape(arg)
         self._shapes[i] = arg
      else:
         self._check_init_shapes(arg)
         self._shapes[i.start : i.stop] = arg

   ## operators ##
   ## these are destructive transformations ##

   def __add__(self, arg):
      return Group([s.__add__(arg) for s in self])

   def __radd__(self, arg):
      return self + arg

   def __iadd__(self, arg):
      self._shapes = [s.__iadd__(arg) for s in self]
      return self

   def __mul__(self, arg):
      return Group([s.__mul__(arg) for s in self])

   def __rmul__(self, arg):
      return self * arg

   def __imul__(self, arg):
      self._shapes = [s.__imul__(arg) for s in self]
      return self

   def __sub__(self, arg):
      return self + (-arg)
   
   def __rsub__(self, arg):
      return (-self) + arg

   def __isub__(self, arg):
      self._shapes = [s.__isub__(arg) for s in self]
      return self

   def __neg__(self):
      self._shapes = [s.__neg__( ) for s in self]
      return self

   def __eq__(self, arg):
      try:
         return self._shapes == arg._shapes
      except AttributeError:
         return False




from chiplotle.geometry.core.path import Path
import math

def spiral_archimedean(radius, num_turns = 5, wrapping_constant = 1, direction = "cw", segments = 500):  
   '''
   Constructs an Archimedean (arithmetic) spiral with the given number of
   turns using the specified number of points.
   
   wrapping_constant controls how tightly the spiral is wound. Several classic
   spirals can be created using different wrapping_constants:
   
   lituus:              -2
   hyperbolic spiral:   -1
   Archimedes' spiral:   1
   Fermat's spiral:      2
   
   scaler controls how large the spiral is.
   
   The general Archimedean spiral equation is:
   
   r = a * theta^(1/n) 
   
   where r is the radius, a is the scaler, and n is the wrapping_constant.
   
   
   More info:
   http://mathworld.wolfram.com/ArchimedeanSpiral.html
   
   '''
   
   two_pi = math.pi * 2.0
   total_rads = two_pi * num_turns
   theta = 0.0
   theta_incr = total_rads / float(segments - 1)
   exponent = 1.0/wrapping_constant
   
   scaler = float(radius) / math.pow(total_rads, exponent)
   
   #Spirals with a negative wrapping_constant technically begin at infinity,
   #which obviously isn't practical. So we nudge theta by a bit to get a
   #reasonable starting point
   if wrapping_constant < 0.0:
      theta += theta_incr
      pow = math.pow(theta_incr, abs(exponent))
      scaler = float(radius) / (1.0/pow)

   spiral_points = []
   r = 0.0
   
   for i in range(segments):
      if exponent > 0:
         r = scaler * math.pow(theta, exponent)
      else:
         pow = math.pow(theta, abs(exponent))
         r = scaler * 1.0/pow
         
      x = math.cos(theta) * r
      y = math.sin(theta) * r
      
      if direction == "ccw":
         y *= -1.0
      #print "r: %f theta: %f" % (r, theta)
      spiral_points.append((x, y))
      
      theta += theta_incr   
   
   result = Path(spiral_points)

   return result


## RUN DEMO CODE

if __name__ == '__main__':
   from chiplotle.geometry.core.group import Group
   from chiplotle.geometry.transforms.offset import offset
   from chiplotle.geometry.transforms.rotate import rotate
   from chiplotle.tools import io
   
   #one of each main spiral type
   s1 = spiral_archimedean(500, wrapping_constant = 1)
   
   s2 = spiral_archimedean(500, wrapping_constant = 2, direction = "ccw")
   offset(s2, (0, -1000))
   
   #these two are long, so we'll rotate them and move them to the side
   #of the others
   s3 = spiral_archimedean(1800, wrapping_constant = -1, direction = "ccw")
   rotate(s3, math.pi * 1.5)
   offset(s3, (650, 400))
   s4 = spiral_archimedean(1500, wrapping_constant = -2, direction = "ccw")
   rotate(s4, math.pi * .6)
   offset(s4, (1000, -1100))
   
   g = Group([s1, s2, s3, s4])

   io.view(g)
